@@ -36,7 +36,7 @@ int CephxServiceHandler::start_session(EntityName& name, bufferlist::iterator& i
   get_random_bytes((char *)&server_challenge, sizeof(server_challenge));
   if (!server_challenge)
     server_challenge = 1;  // always non-zero.
-  dout(10) << "start_session server_challenge " << hex << server_challenge << dec << dendl;
+  ldout(cct, 10) << "start_session server_challenge " << hex << server_challenge << dec << dendl;
 
   CephXServerChallenge ch;
   ch.server_challenge = server_challenge;
@@ -55,14 +55,14 @@ int CephxServiceHandler::handle_request(bufferlist::iterator& indata, bufferlist
   switch (cephx_header.request_type) {
   case CEPHX_GET_AUTH_SESSION_KEY:
     {
-      dout(10) << "handle_request get_auth_session_key for " << entity_name << dendl;
+      ldout(cct, 10) << "handle_request get_auth_session_key for " << entity_name << dendl;
 
       CephXAuthenticate req;
       ::decode(req, indata);
 
       CryptoKey secret;
       if (!key_server->get_secret(entity_name, secret)) {
-        dout(0) << "couldn't find entity name: " << entity_name << dendl;
+        ldout(cct, 0) << "couldn't find entity name: " << entity_name << dendl;
 	ret = -EPERM;
 	break;
       }
@@ -73,12 +73,19 @@ int CephxServiceHandler::handle_request(bufferlist::iterator& indata, bufferlist
       }      
 
       uint64_t expected_key;
-      cephx_calc_client_server_challenge(secret, server_challenge, req.client_challenge, &expected_key);
+      std::string error;
+      cephx_calc_client_server_challenge(secret, server_challenge,
+					 req.client_challenge, &expected_key, error);
+      if (!error.empty()) {
+	ldout(cct, 0) << " cephx_calc_client_server_challenge error: " << error << dendl;
+	ret = -EPERM;
+	break;
+      }
 
-      dout(20) << " checking key: req.key=" << hex << req.key
+      ldout(cct, 20) << " checking key: req.key=" << hex << req.key
 	       << " expected_key=" << expected_key << dec << dendl;
       if (req.key != expected_key) {
-        dout(0) << " unexpected key: req.key=" << hex << req.key
+        ldout(cct, 0) << " unexpected key: req.key=" << hex << req.key
 		<< " expected_key=" << expected_key << dec << dendl;
         ret = -EPERM;
 	break;
@@ -95,17 +102,18 @@ int CephxServiceHandler::handle_request(bufferlist::iterator& indata, bufferlist
       }
       CephXServiceTicketInfo old_ticket_info;
 
-      if (cephx_decode_ticket(key_server, CEPH_ENTITY_TYPE_AUTH, req.old_ticket, old_ticket_info)) {
+      if (cephx_decode_ticket(cct, key_server, CEPH_ENTITY_TYPE_AUTH,
+			      req.old_ticket, old_ticket_info)) {
         global_id = old_ticket_info.ticket.global_id;
-        dout(10) << "decoded old_ticket with global_id=" << global_id << dendl;
+        ldout(cct, 10) << "decoded old_ticket with global_id=" << global_id << dendl;
         should_enc_ticket = true;
       }
 
-      info.ticket.init_timestamps(g_clock.now(), g_conf->auth_mon_ticket_ttl);
+      info.ticket.init_timestamps(ceph_clock_now(cct), cct->_conf->auth_mon_ticket_ttl);
       info.ticket.name = entity_name;
       info.ticket.global_id = global_id;
       info.ticket.auid = eauth.auid;
-      info.validity += g_conf->auth_mon_ticket_ttl;
+      info.validity += cct->_conf->auth_mon_ticket_ttl;
 
       if (auid) *auid = eauth.auid;
 
@@ -114,7 +122,7 @@ int CephxServiceHandler::handle_request(bufferlist::iterator& indata, bufferlist
       info.session_key = session_key;
       info.service_id = CEPH_ENTITY_TYPE_AUTH;
       if (!key_server->get_service_secret(CEPH_ENTITY_TYPE_AUTH, info.service_secret, info.secret_id)) {
-        dout(0) << " could not get service secret for auth subsystem" << dendl;
+        ldout(cct, 0) << " could not get service secret for auth subsystem" << dendl;
         ret = -EIO;
         break;
       }
@@ -123,56 +131,56 @@ int CephxServiceHandler::handle_request(bufferlist::iterator& indata, bufferlist
       info_vec.push_back(info);
 
       build_cephx_response_header(cephx_header.request_type, 0, result_bl);
-      if (!cephx_build_service_ticket_reply(eauth.key, info_vec, should_enc_ticket, old_ticket_info.session_key, result_bl)) {
-        ret = -EIO;
-        break;
+      if (!cephx_build_service_ticket_reply(cct, eauth.key, info_vec, should_enc_ticket,
+					    old_ticket_info.session_key, result_bl)) {
+	ret = -EIO;
       }
 
       if (!key_server->get_service_caps(entity_name, CEPH_ENTITY_TYPE_MON, caps)) {
-        dout(0) << " could not get mon caps for " << entity_name << dendl;
+        ldout(cct, 0) << " could not get mon caps for " << entity_name << dendl;
       }
     }
     break;
 
   case CEPHX_GET_PRINCIPAL_SESSION_KEY:
     {
-      dout(10) << "handle_request get_principal_session_key" << dendl;
+      ldout(cct, 10) << "handle_request get_principal_session_key" << dendl;
 
       bufferlist tmp_bl;
       CephXServiceTicketInfo auth_ticket_info;
-      if (!cephx_verify_authorizer(key_server, indata, auth_ticket_info, tmp_bl)) {
+      if (!cephx_verify_authorizer(cct, key_server, indata, auth_ticket_info, tmp_bl)) {
         ret = -EPERM;
 	break;
       }
 
       CephXServiceTicketRequest ticket_req;
       ::decode(ticket_req, indata);
-      dout(10) << " ticket_req.keys = " << ticket_req.keys << dendl;
+      ldout(cct, 10) << " ticket_req.keys = " << ticket_req.keys << dendl;
 
       ret = 0;
       vector<CephXSessionAuthInfo> info_vec;
       for (uint32_t service_id = 1; service_id <= ticket_req.keys; service_id <<= 1) {
         if (ticket_req.keys & service_id) {
-	  dout(10) << " adding key for service " << ceph_entity_type_name(service_id) << dendl;
+	  ldout(cct, 10) << " adding key for service " << ceph_entity_type_name(service_id) << dendl;
           CephXSessionAuthInfo info;
           int r = key_server->build_session_auth_info(service_id, auth_ticket_info, info);
           if (r < 0) {
             ret = r;
             break;
           }
-          info.validity += g_conf->auth_service_ticket_ttl;
+          info.validity += cct->_conf->auth_service_ticket_ttl;
           info_vec.push_back(info);
         }
       }
       CryptoKey no_key;
       build_cephx_response_header(cephx_header.request_type, ret, result_bl);
-      cephx_build_service_ticket_reply(auth_ticket_info.session_key, info_vec, false, no_key, result_bl);
+      cephx_build_service_ticket_reply(cct, auth_ticket_info.session_key, info_vec, false, no_key, result_bl);
     }
     break;
 
   case CEPHX_GET_ROTATING_KEY:
     {
-      dout(10) << "handle_request getting rotating secret for " << entity_name << dendl;
+      ldout(cct, 10) << "handle_request getting rotating secret for " << entity_name << dendl;
       build_cephx_response_header(cephx_header.request_type, 0, result_bl);
       key_server->get_rotating_encrypted(entity_name, result_bl);
       ret = 0;
@@ -180,7 +188,7 @@ int CephxServiceHandler::handle_request(bufferlist::iterator& indata, bufferlist
     break;
 
   default:
-    dout(10) << "handle_request unknown op " << cephx_header.request_type << dendl;
+    ldout(cct, 10) << "handle_request unknown op " << cephx_header.request_type << dendl;
     return -EINVAL;
   }
   return ret;
