@@ -64,7 +64,9 @@ static int do_cmds_special_action(const std::string &action,
 				  const std::string &dump_file, int rank)
 {
   SimpleMessenger *messenger = new SimpleMessenger(g_ceph_context);
-  messenger->bind(g_conf->public_addr, getpid());
+  int r = messenger->bind(g_conf->public_addr, getpid());
+  if (r < 0)
+    return r;
   MonClient mc(g_ceph_context);
   if (mc.build_initial_monmap() < 0)
     return -1;
@@ -211,7 +213,12 @@ int main(int argc, const char **argv)
   global_print_banner();
 
   SimpleMessenger *messenger = new SimpleMessenger(g_ceph_context);
-  messenger->bind(g_conf->public_addr, getpid());
+  messenger->set_cluster_protocol(CEPH_MDS_PROTOCOL);
+
+  int r = messenger->bind(g_conf->public_addr, getpid());
+  if (r < 0)
+    exit(1);
+
   cout << "starting " << g_conf->name << " at " << messenger->get_ms_addr()
        << std::endl;
   messenger->register_entity(entity_name_t::MDS(-1));
@@ -220,7 +227,9 @@ int main(int argc, const char **argv)
     CEPH_FEATURE_NOSRCADDR |
     CEPH_FEATURE_DIRLAYOUTHASH |
     CEPH_FEATURE_PGID64;
-  messenger->set_default_policy(SimpleMessenger::Policy::client(supported, 0));
+  uint64_t required =
+    CEPH_FEATURE_OSDREPLYMUX;
+  messenger->set_default_policy(SimpleMessenger::Policy::client(supported, required));
   messenger->set_policy(entity_name_t::TYPE_MON,
 			SimpleMessenger::Policy::client(supported,
 							CEPH_FEATURE_UID |
