@@ -67,7 +67,7 @@ class Filer;
 class Objecter;
 class ObjectCacher;
 
-extern class PerfCounters *client_counters;
+class PerfCounters;
 
 enum {
   l_c_first = 20000,
@@ -185,6 +185,8 @@ class Client : public Dispatcher {
  public:
   CephContext *cct;
 
+  PerfCounters *logger;
+
   // cluster descriptors
   MDSMap *mdsmap; 
   OSDMap *osdmap;
@@ -300,8 +302,6 @@ protected:
   //  - protects Client and buffer cache both!
   Mutex                  client_lock;
 
-  int filer_flags;
-
   // helpers
   void wake_inode_waiters(int mds);
   void wait_on_list(list<Cond*>& ls);
@@ -369,7 +369,7 @@ protected:
 
   inodeno_t get_root_ino();
 
-  void init();
+  int init();
   void shutdown();
 
   // messaging
@@ -524,7 +524,7 @@ public:
    * and the offset.
    *
    * Returns 0 if it reached the end of the directory.
-   * If @cb returns a negative error code, stop and return that.
+   * If @a cb returns a negative error code, stop and return that.
    */
   int readdir_r_cb(dir_result_t *dirp, add_dirent_cb_t cb, void *p);
 
