@@ -13,27 +13,25 @@
  */
 
 
+#include "common/Clock.h"
+#include "common/ceph_context.h"
 #include "common/config.h"
-#include "Clock.h"
+#include "include/utime.h"
 
-// public
-Clock g_clock;
+#include <time.h>
 
-Clock::Clock() {
-}
-
-Clock::~Clock() {
-}
-
-utime_t Clock::now() {
+utime_t ceph_clock_now(CephContext *cct)
+{
   struct timeval tv;
   gettimeofday(&tv, NULL);
   utime_t n(&tv);
-  n += g_conf->clock_offset;
-  if (n < last) {
-    //derr << "WARNING: clock jumped backwards from " << last << " to " << n << dendl;
-    n = last;    // clock jumped backwards!
-  } else
-    last = n;
+  n += cct->_conf->clock_offset;
   return n;
 }
+
+time_t ceph_clock_gettime(CephContext *cct)
+{
+  time_t ret = time(NULL);
+  ret += ((time_t)cct->_conf->clock_offset);
+  return ret;
+}
