@@ -39,8 +39,6 @@ inline static void decode(crush_rule_step &s, bufferlist::iterator &p)
   ::decode(s.arg2, p);
 }
 
-
-
 using namespace std;
 class CrushWrapper {
 public:
@@ -161,17 +159,17 @@ public:
 
 
   void find_roots(set<int>& roots) const;
-  int insert_item(int id, float weight, string name, map<string,string>& loc);
-  int remove_item(int id);
-  int adjust_item_weight(int id, int weight);
-  int adjust_item_weightf(int id, float weight) {
-    return adjust_item_weight(id, (int)(weight * (float)0x10000));
+  int insert_item(CephContext *cct, int id, float weight, string name, map<string,string>& loc);
+  int remove_item(CephContext *cct, int id);
+  int adjust_item_weight(CephContext *cct, int id, int weight);
+  int adjust_item_weightf(CephContext *cct, int id, float weight) {
+    return adjust_item_weight(cct, id, (int)(weight * (float)0x10000));
   }
-  void reweight();
+  void reweight(CephContext *cct);
 
 
   /*** devices ***/
-  int get_max_devices() {
+  int get_max_devices() const {
     if (!crush) return 0;
     return crush->max_devices;
   }
@@ -382,12 +380,12 @@ public:
     crush->max_devices = m;
   }
 
-  int find_rule(int ruleset, int type, int size) {
+  int find_rule(int ruleset, int type, int size) const {
     if (!crush) return -1;
     return crush_find_rule(crush, ruleset, type, size);
   }
   void do_rule(int rule, int x, vector<int>& out, int maxout, int forcefeed,
-	       vector<__u32>& weight) {
+	       const vector<__u32>& weight) const {
     int rawout[maxout];
     int numrep = crush_do_rule(crush, rule, x, rawout, maxout,
 			       forcefeed, &weight[0]);
