@@ -1,5 +1,5 @@
-#ifndef _CEPH_NSTRING
-#define _CEPH_NSTRING
+#ifndef CEPH_NSTRING_D44FAD468A084A2D994D171C04DE05BD
+#define CEPH_NSTRING_D44FAD468A084A2D994D171C04DE05BD
 
 #if 0
 # include "tstring.h"
