@@ -28,6 +28,7 @@ using namespace __gnu_cxx;
 #include "hash.h"
 #include "encoding.h"
 #include "ceph_hash.h"
+#include "cmp.h"
 
 /* Maximum supported object name length for Ceph, in bytes.
  *
@@ -43,9 +44,13 @@ struct object_t {
   object_t() {}
   object_t(const char *s) : name(s) {}
   object_t(const string& s) : name(s) {}
+
   void swap(object_t& o) {
     name.swap(o.name);
   }
+  void clear() {
+    name.clear();
+  }
   
   void encode(bufferlist &bl) const {
     ::encode(name, bl);
@@ -275,6 +280,7 @@ public:
   }
   
   hobject_t() : snap(0), hash(0), max(false) {}
+
   hobject_t(object_t oid, const string& key, snapid_t snap, uint64_t hash) : 
     oid(oid), snap(snap), hash(hash), max(false),
     key(oid.name == key ? string() : key) {}
@@ -283,6 +289,12 @@ public:
     oid(soid.oid), snap(soid.snap), hash(hash), max(false),
     key(soid.oid.name == key ? string() : key) {}
 
+  /* Do not use when a particular hash function is needed */
+  explicit hobject_t(const sobject_t &o) :
+    oid(o.oid), snap(o.snap), max(false) {
+    hash = __gnu_cxx::hash<sobject_t>()(o);
+  }
+
   // maximum sorted value.
   static hobject_t get_max() {
     hobject_t h;
@@ -311,10 +323,23 @@ public:
     hash = _reverse_nibbles(v);
   }
 
-  /* Do not use when a particular hash function is needed */
-  explicit hobject_t(const sobject_t &o) :
-    oid(o.oid), snap(o.snap), max(false) {
-    hash = __gnu_cxx::hash<sobject_t>()(o);
+  const string& get_effective_key() const {
+    if (key.length())
+      return key;
+    return oid.name;
+  }
+
+  /**
+   * back up hobject_t to beginning of hash bucket, if i am partway through one.
+   */
+  void back_up_to_bounding_key() {
+    if (key.length()) {
+      oid.clear();
+    } else {
+      key = oid.name;
+      oid.clear();
+    }
+    snap = 0;
   }
 
   void swap(hobject_t &o) {
@@ -368,43 +393,16 @@ inline ostream& operator<<(ostream& out, const hobject_t& o)
 {
   if (o.is_max())
     return out << "MAX";
-  out << o.oid << "/" << o.snap << "/" << std::hex << o.hash << std::dec;
+  out << std::hex << o.hash << std::dec;
   if (o.get_key().length())
-    out << "@" << o.get_key();
+    out << "." << o.get_key();
+  out << "/" << o.oid << "/" << o.snap;
   return out;
 }
 
-// sort hobject_t's by <get_filestore_key,name,snapid>
-inline bool operator==(const hobject_t &l, const hobject_t &r) {
-  return l.oid == r.oid && l.snap == r.snap && l.hash == r.hash && l.max == r.max;
-}
-inline bool operator!=(const hobject_t &l, const hobject_t &r) {
-  return l.oid != r.oid || l.snap != r.snap || l.hash != r.hash || l.max != r.max;
-}
-inline bool operator>(const hobject_t &l, const hobject_t &r) {
-  return l.max > r.max ||
-    (l.max == r.max && (l.get_filestore_key() > r.get_filestore_key() ||
-			(l.get_filestore_key() == r.get_filestore_key() && (l.oid > r.oid || 
-					      (l.oid == r.oid && l.snap > r.snap)))));
-}
-inline bool operator<(const hobject_t &l, const hobject_t &r) {
-  return l.max < r.max ||
-    (l.max == r.max && (l.get_filestore_key() < r.get_filestore_key() ||
-			(l.get_filestore_key() == r.get_filestore_key() && (l.oid < r.oid ||
-					      (l.oid == r.oid && l.snap < r.snap)))));
-}
-inline bool operator>=(const hobject_t &l, const hobject_t &r) {
-  return l.max > r.max ||
-    (l.max == r.max && (l.get_filestore_key() > r.get_filestore_key() ||
-			(l.get_filestore_key() == r.get_filestore_key() && (l.oid > r.oid ||
-					      (l.oid == r.oid && l.snap >= r.snap)))));
-}
-inline bool operator<=(const hobject_t &l, const hobject_t &r) {
-  return l.max < r.max ||
-    (l.max == r.max && (l.get_filestore_key() < r.get_filestore_key() ||
-			(l.get_filestore_key() == r.get_filestore_key() && (l.oid < r.oid ||
-					      (l.oid == r.oid && l.snap <= r.snap)))));
-}
+WRITE_EQ_OPERATORS_5(hobject_t, oid, get_key(), snap, hash, max)
+// sort hobject_t's by <max, get_filestore_key(hash), key, oid, snapid>
+WRITE_CMP_OPERATORS_5(hobject_t, max, get_filestore_key(), get_effective_key(), oid, snap)
 
 
 // ---------------------------
