@@ -41,6 +41,13 @@ namespace librbd {
 
   typedef rbd_image_info_t image_info_t;
 
+  class ProgressContext
+  {
+  public:
+    virtual ~ProgressContext();
+    virtual int update_progress(uint64_t offset, uint64_t total) = 0;
+  };
+
 class RBD
 {
 public:
@@ -62,7 +69,7 @@ public:
   int list(IoCtx& io_ctx, std::vector<std::string>& names);
   int create(IoCtx& io_ctx, const char *name, uint64_t size, int *order);
   int remove(IoCtx& io_ctx, const char *name);
-  int copy(IoCtx& src_io_ctx, const char *srcname, IoCtx& dest_io_ctx, const char *destname);
+  int remove_with_progress(IoCtx& io_ctx, const char *name, ProgressContext& pctx);
   int rename(IoCtx& src_io_ctx, const char *srcname, const char *destname);
 
 private:
@@ -78,13 +85,18 @@ public:
   ~Image();
 
   int resize(uint64_t size);
+  int resize_with_progress(uint64_t size, ProgressContext& pctx);
   int stat(image_info_t &info, size_t infosize);
+  int copy(IoCtx& dest_io_ctx, const char *destname);
+  int copy_with_progress(IoCtx& dest_io_ctx, const char *destname,
+			 ProgressContext &prog_ctx);
 
   /* snapshots */
   int snap_list(std::vector<snap_info_t>& snaps);
   int snap_create(const char *snapname);
   int snap_remove(const char *snapname);
   int snap_rollback(const char *snap_name);
+  int snap_rollback_with_progress(const char *snap_name, ProgressContext& pctx);
   int snap_set(const char *snap_name);
 
   /* I/O */
@@ -96,6 +108,8 @@ public:
   int aio_write(uint64_t off, size_t len, ceph::bufferlist& bl, RBD::AioCompletion *c);
   int aio_read(uint64_t off, size_t len, ceph::bufferlist& bl, RBD::AioCompletion *c);
 
+  int flush();
+
 private:
   friend class RBD;
 
