@@ -82,13 +82,6 @@
 
 
 
-#ifdef ENABLE_COVERAGE
-void handle_signal(int signal)
-{
-  exit(0);
-}
-#endif
-
 // cons/des
 MDS::MDS(const std::string &n, Messenger *m, MonClient *mc) : 
   Dispatcher(m->cct),
@@ -139,7 +132,6 @@ MDS::MDS(const std::string &n, Messenger *m, MonClient *mc) :
   // beacon
   beacon_last_seq = 0;
   beacon_sender = 0;
-  beacon_killer = 0;
   was_laggy = false;
 
   // tick
@@ -512,10 +504,6 @@ int MDS::init(int wanted_state)
 
   mds_lock.Unlock();
 
-#ifdef ENABLE_COVERAGE
-  signal(SIGTERM, handle_signal);
-#endif
-
   return 0;
 }
 
@@ -635,6 +623,12 @@ bool MDS::is_laggy()
     dout(5) << "is_laggy " << since << " > " << g_conf->mds_beacon_grace
 	    << " since last acked beacon" << dendl;
     was_laggy = true;
+    if (since > (g_conf->mds_beacon_grace*2)) {
+      // maybe it's not us?
+      dout(5) << "initiating monitor reconnect; maybe we're not the slow one"
+              << dendl;
+      monc->reopen_session();
+    }
     return true;
   }
   return false;
@@ -667,8 +661,6 @@ void MDS::handle_mds_beacon(MMDSBeacon *m)
     while (!beacon_seq_stamp.empty() &&
 	   beacon_seq_stamp.begin()->first <= seq)
       beacon_seq_stamp.erase(beacon_seq_stamp.begin());
-    
-    reset_beacon_killer();
   } else {
     dout(10) << "handle_mds_beacon " << ceph_mds_state_name(m->get_state())
 	     << " seq " << m->get_seq() << " dne" << dendl;
@@ -677,35 +669,6 @@ void MDS::handle_mds_beacon(MMDSBeacon *m)
   m->put();
 }
 
-void MDS::reset_beacon_killer()
-{
-  utime_t when = beacon_last_acked_stamp;
-  when += g_conf->mds_beacon_grace;
-  
-  dout(25) << "reset_beacon_killer last_acked_stamp at " << beacon_last_acked_stamp
-	   << ", will die at " << when << dendl;
-  
-  if (beacon_killer) timer.cancel_event(beacon_killer);
-
-  beacon_killer = new C_MDS_BeaconKiller(this, beacon_last_acked_stamp);
-  timer.add_event_at(when, beacon_killer);
-}
-
-void MDS::beacon_kill(utime_t lab)
-{
-  if (lab == beacon_last_acked_stamp) {
-    dout(0) << "beacon_kill last_acked_stamp " << lab 
-	    << ", we are laggy!"
-	    << dendl;
-    //suicide();
-  } else {
-    dout(20) << "beacon_kill last_acked_stamp " << beacon_last_acked_stamp 
-	     << " != my " << lab 
-	     << ", doing nothing."
-	     << dendl;
-  }
-}
-
 /* This function DOES put the passed message before returning*/
 void MDS::handle_command(MMonCommand *m)
 {
@@ -1386,8 +1349,8 @@ void MDS::replay_done()
     inotable->save(new C_NoopContext);
   }
 
-  if (mdsmap->get_num_mds() == 1 &&
-      mdsmap->get_num_failed() == 0) { // just me!
+  if (mdsmap->get_num_in_mds() == 1 &&
+      mdsmap->get_num_failed_mds() == 0) { // just me!
     dout(2) << "i am alone, moving to state reconnect" << dendl;      
     request_state(MDSMap::STATE_RECONNECT);
   } else {
@@ -1531,7 +1494,7 @@ void MDS::stopping_start()
 {
   dout(2) << "stopping_start" << dendl;
 
-  if (mdsmap->get_num_mds() == 1 && !sessionmap.empty()) {
+  if (mdsmap->get_num_in_mds() == 1 && !sessionmap.empty()) {
     // we're the only mds up!
     dout(0) << "we are the last MDS, and have mounted clients: we cannot flush our journal.  suicide!" << dendl;
     suicide();
@@ -1562,10 +1525,6 @@ void MDS::suicide()
 	  << ", now " << ceph_mds_state_name(state) << dendl;
 
   // stop timers
-  if (beacon_killer) {
-    timer.cancel_event(beacon_killer);
-    beacon_killer = 0;
-  }
   if (beacon_sender) {
     timer.cancel_event(beacon_sender);
     beacon_sender = 0;
