@@ -20,17 +20,19 @@
 struct MAuth : public PaxosServiceMessage {
   __u32 protocol;
   bufferlist auth_payload;
+  epoch_t monmap_epoch;
 
   /* if protocol == 0, then auth_payload is a set<__u32> listing protocols the client supports */
 
-  MAuth() : PaxosServiceMessage(CEPH_MSG_AUTH, 0), protocol(0) { }
+  MAuth() : PaxosServiceMessage(CEPH_MSG_AUTH, 0), protocol(0), monmap_epoch(0) { }
 private:
   ~MAuth() {}
 
 public:
   const char *get_type_name() { return "auth"; }
   void print(ostream& out) {
-    out << "auth(proto " << protocol << " " << auth_payload.length() << " bytes)";
+    out << "auth(proto " << protocol << " " << auth_payload.length() << " bytes"
+	<< " epoch " << monmap_epoch << ")";
   }
 
   void decode_payload(CephContext *cct) {
@@ -38,11 +40,16 @@ public:
     paxos_decode(p);
     ::decode(protocol, p);
     ::decode(auth_payload, p);
+    if (!p.end())
+      ::decode(monmap_epoch, p);
+    else
+      monmap_epoch = 0;
   }
   void encode_payload(CephContext *cct) {
     paxos_encode();
     ::encode(protocol, payload);
     ::encode(auth_payload, payload);
+    ::encode(monmap_epoch, payload);
   }
   bufferlist& get_auth_payload() { return auth_payload; }
 };
