@@ -41,13 +41,13 @@ public:
     o << "dentry_unlink(" << dirfrag << " " << dn << ")";
   }
   
-  void decode_payload() {
+  void decode_payload(CephContext *cct) {
     bufferlist::iterator p = payload.begin();
     ::decode(dirfrag, p);
     ::decode(dn, p);
     ::decode(straybl, p);
   }
-  void encode_payload() {
+  void encode_payload(CephContext *cct) {
     ::encode(dirfrag, payload);
     ::encode(dn, payload);
     ::encode(straybl, payload);
