@@ -66,7 +66,7 @@ class MMonPaxos : public Message {
     op(o), machine_id(mid),
     first_committed(0), last_committed(0), pn_from(0), pn(0), uncommitted_pn(0),
     latest_version(0) {
-    sent_timestamp = g_clock.now();
+    sent_timestamp = ceph_clock_now(g_ceph_context);
   }
 
 private:
@@ -86,7 +86,7 @@ public:
     out <<  ")";
   }
 
-  void encode_payload() {
+  void encode_payload(CephContext *cct) {
     if (connection->has_feature(CEPH_FEATURE_MONCLOCKCHECK))
       header.version = 1;
     ::encode(epoch, payload);
@@ -104,7 +104,7 @@ public:
     ::encode(latest_value, payload);
     ::encode(values, payload);
   }
-  void decode_payload() {
+  void decode_payload(CephContext *cct) {
     bufferlist::iterator p = payload.begin();
     ::decode(epoch, p);
     ::decode(op, p);
