@@ -38,6 +38,7 @@ class MMonSubscribeAck;
 class MAuthReply;
 class MAuthRotating;
 class LogClient;
+class AuthSupported;
 
 enum MonClientState {
   MC_STATE_NONE,
@@ -68,7 +69,7 @@ private:
 
   LogClient *log_client;
 
-  set<__u32> auth_supported;
+  AuthSupported *auth_supported;
 
   bool ms_dispatch(Message *m);
   bool ms_handle_reset(Connection *con);
@@ -199,7 +200,7 @@ public:
     return my_addr;
   }
 
-  const ceph_fsid_t& get_fsid() {
+  const uuid_d& get_fsid() {
     return monmap.fsid;
   }
 
