@@ -207,7 +207,7 @@ void Monitor::init()
     pcb.add_u64(l_cluster_num_object, "num_object");
     pcb.add_u64(l_cluster_num_object_degraded, "num_object_degraded");
     pcb.add_u64(l_cluster_num_object_unfound, "num_object_unfound");
-    pcb.add_u64(l_cluster_num_kb, "num_kb");
+    pcb.add_u64(l_cluster_num_bytes, "num_bytes");
     pcb.add_u64(l_cluster_num_mds_up, "num_mds_up");
     pcb.add_u64(l_cluster_num_mds_in, "num_mds_in");
     pcb.add_u64(l_cluster_num_mds_failed, "num_mds_failed");
