@@ -44,6 +44,10 @@
 
 #include <memory>
 
+
+#define CEPH_MON_PROTOCOL     9 /* cluster internal */
+
+
 enum {
   l_cluster_first = 555000,
   l_cluster_num_mon,
@@ -63,7 +67,7 @@ enum {
   l_cluster_num_object,
   l_cluster_num_object_degraded,
   l_cluster_num_object_unfound,
-  l_cluster_num_kb,
+  l_cluster_num_bytes,
   l_cluster_num_mds_up,
   l_cluster_num_mds_in,
   l_cluster_num_mds_failed,
