@@ -17,6 +17,8 @@
 #include "MonitorStore.h"
 
 #include "messages/MMonCommand.h"
+#include "messages/MMonJoin.h"
+
 #include "common/Timer.h"
 #include "common/ceph_argparse.h"
 #include "mon/MDSMonitor.h"
@@ -31,56 +33,59 @@
 #define dout_prefix _prefix(_dout, mon)
 static ostream& _prefix(std::ostream *_dout, Monitor *mon) {
   return *_dout << "mon." << mon->name << "@" << mon->rank
-		<< (mon->is_starting() ? (const char*)"(starting)":(mon->is_leader() ? (const char*)"(leader)":(mon->is_peon() ? (const char*)"(peon)":(const char*)"(?\?)")))
-		<< ".monmap v" << mon->monmap->epoch << " ";
+		<< "(" << mon->get_state_name()
+		<< ").monmap v" << mon->monmap->epoch << " ";
 }
 
-void MonmapMonitor::create_initial(bufferlist& bl)
+void MonmapMonitor::create_initial()
 {
+  bufferlist bl;
+  mon->store->get_bl_ss(bl, "mkfs", "monmap");
   pending_map.decode(bl);
-  dout(10) << "create_initial was fed epoch " << pending_map.epoch << dendl;
+  dout(10) << "create_initial set fed epoch " << pending_map.epoch << dendl;
+  assert(pending_map.epoch == 0);   // fix mkfs()
+  pending_map.epoch = 1;
 }
 
 bool MonmapMonitor::update_from_paxos()
 {
-  //check versions to see if there's an update
   version_t paxosv = paxos->get_version();
-  if (paxosv <= mon->monmap->epoch) return true;
-  //assert(paxosv >= mon->monmap->epoch);
+  if (paxosv <= paxos->get_stashed_version() &&
+      paxosv <= mon->monmap->get_epoch())
+    return true;
 
   dout(10) << "update_from_paxos paxosv " << paxosv
 	   << ", my v " << mon->monmap->epoch << dendl;
-
-  //read and decode
-  monmap_bl.clear();
-  bool success = paxos->read(paxosv, monmap_bl);
-  assert(success);
-  dout(10) << "update_from_paxos got " << paxosv << dendl;
-  mon->monmap->decode(monmap_bl);
-
-  //save the bufferlist version in the paxos instance as well
-  paxos->stash_latest(paxosv, monmap_bl);
-
-  int rank = mon->monmap->get_rank(mon->name);
-  if (rank < 0) {
-    dout(10) << "Assuming temporary id=mon." << mon->monmap->size() << " for shutdown purposes" << dendl;
-    mon->messenger->set_myname(entity_name_t::MON(mon->monmap->size()));
-    mon->monmap->add(mon->name, mon->myaddr);
-    mon->shutdown();
-    return true;
+  
+  version_t orig_latest = paxos->get_stashed_version();
+  bool need_restart = paxosv != mon->monmap->get_epoch();  
+  
+  if (paxosv > 0 && (mon->monmap->get_epoch() == 0 ||
+		     paxos->get_stashed_version() != paxosv)) {
+    bufferlist latest;
+    version_t v = paxos->get_stashed(latest);
+    if (v) {
+      mon->monmap->decode(latest);
+    }
   }
 
-  if (rank != mon->rank) {
-    mon->messenger->set_myname(entity_name_t::MON(rank));
-    mon->rank = rank;
+  if (paxosv > mon->monmap->get_epoch()) {
+    // read and decode
+    monmap_bl.clear();
+    bool success = paxos->read(paxosv, monmap_bl);
+    assert(success);
+    dout(10) << "update_from_paxos got " << paxosv << dendl;
+    mon->monmap->decode(monmap_bl);
+
+    // save the bufferlist version in the paxos instance as well
+    paxos->stash_latest(paxosv, monmap_bl);
+
+    if (orig_latest == 0)
+      mon->store->erase_ss("mkfs", "monmap");
   }
 
-  // call election?
-  if (mon->monmap->size() > 1) {
-    mon->call_election();
-  } else {
-    mon->win_standalone_election();
-  }
+  if (need_restart)
+    mon->bootstrap();
   return true;
 }
 
@@ -107,6 +112,8 @@ bool MonmapMonitor::preprocess_query(PaxosServiceMessage *m)
     // READs
   case MSG_MON_COMMAND:
     return preprocess_command((MMonCommand*)m);
+  case MSG_MON_JOIN:
+    return preprocess_join((MMonJoin*)m);
   default:
     assert(0);
     m->put();
@@ -247,6 +254,8 @@ bool MonmapMonitor::prepare_update(PaxosServiceMessage *m)
   switch (m->get_type()) {
   case MSG_MON_COMMAND:
     return prepare_command((MMonCommand*)m);
+  case MSG_MON_JOIN:
+    return prepare_join((MMonJoin*)m);
   default:
     assert(0);
     m->put();
@@ -313,17 +322,37 @@ out:
   return false;
 }
 
+bool MonmapMonitor::preprocess_join(MMonJoin *join)
+{
+  dout(10) << "preprocess_join " << join->name << " at " << join->addr << dendl;
+
+  if (pending_map.contains(join->name)) {
+    dout(10) << " already have " << join->name << dendl;
+    join->put();
+    return true;
+  }
+  if (pending_map.contains(join->addr)) {
+    dout(10) << " already have " << join->addr << dendl;
+    join->put();
+    return true;
+  }
+  return false;
+}
+bool MonmapMonitor::prepare_join(MMonJoin *join)
+{
+  dout(0) << "adding " << join->name << " at " << join->addr << " to monitor cluster" << dendl;
+  pending_map.add(join->name, join->addr);
+  pending_map.last_changed = ceph_clock_now(g_ceph_context);
+  join->put();
+  return true;
+}
+
 bool MonmapMonitor::should_propose(double& delay)
 {
   delay = 0.0;
   return true;
 }
 
-void MonmapMonitor::committed()
-{
-  //Nothing useful to do here.
-}
-
 void MonmapMonitor::tick()
 {
   update_from_paxos();
