@@ -56,9 +56,12 @@ private:
   bool update_from_paxos();
   void create_pending();  // prepare a new pending
   void encode_pending(bufferlist &bl);
+  void on_active();
 
   void share_map_with_random_osd();
 
+  void update_logger();
+
   void handle_query(PaxosServiceMessage *m);
   bool preprocess_query(PaxosServiceMessage *m);  // true if processed.
   bool prepare_update(PaxosServiceMessage *m);
@@ -98,7 +101,8 @@ private:
   bool prepare_pool_op_create (MPoolOp *m);
   bool prepare_pool_op_delete(MPoolOp *m);
   bool prepare_pool_op_auid(MPoolOp *m);
-  int prepare_new_pool(string& name, uint64_t auid, int crush_rule);
+  int prepare_new_pool(string& name, uint64_t auid, int crush_rule,
+                       unsigned pg_num, unsigned pgp_num);
   int prepare_new_pool(MPoolOp *m);
 
   void _pool_op_reply(MPoolOp *m, int ret, epoch_t epoch, bufferlist *blp=NULL);
