@@ -65,6 +65,8 @@ using namespace std;
 #include "messages/MOSDPGMissing.h"
 #include "messages/MOSDScrub.h"
 #include "messages/MOSDRepScrub.h"
+#include "messages/MOSDPGScan.h"
+#include "messages/MOSDPGBackfill.h"
 
 #include "messages/MRemoveSnaps.h"
 
@@ -353,6 +355,12 @@ Message *decode_message(CephContext *cct, ceph_msg_header& header, ceph_msg_foot
   case MSG_OSD_REP_SCRUB:
     m = new MOSDRepScrub;
     break;
+  case MSG_OSD_PG_SCAN:
+    m = new MOSDPGScan;
+    break;
+  case MSG_OSD_PG_BACKFILL:
+    m = new MOSDPGBackfill;
+    break;
    // auth
   case CEPH_MSG_AUTH:
     m = new MAuth;
