@@ -15,10 +15,10 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
-#ifdef DARWIN
+#if defined(DARWIN) || defined(__FreeBSD__)
 #include <sys/param.h>
 #include <sys/mount.h>
-#endif // DARWIN
+#endif // DARWIN || __FreeBSD__
 
 
 int myrand() 
