@@ -60,6 +60,19 @@ public:
     void finish(int r);
   };
 
+  class C_WatchTimeout : public Context {
+    OSD *osd;
+    void *obc;
+    void *pg;
+    entity_name_t entity;
+    utime_t expire;
+  public:
+    C_WatchTimeout(OSD *_osd, void *_obc, void *_pg,
+		   entity_name_t _entity, utime_t _expire) :
+      osd(_osd), obc(_obc), pg(_pg), entity(_entity), expire(_expire) {}
+    void finish(int r);
+  };
+
 private:
   std::map<uint64_t, Notification *> notifs; /* notif_id to notifications */
 
