@@ -502,7 +502,9 @@ inline ostream& operator<<(ostream& out, const osd_stat_t& s) {
 #define PG_STATE_INCONSISTENT (1<<11) // pg replicas are inconsistent (but shouldn't be)
 #define PG_STATE_PEERING      (1<<12) // pg is (re)peering
 #define PG_STATE_REPAIR       (1<<13) // pg should repair on next scrub
-#define PG_STATE_SCANNING     (1<<14) // scanning content to generate backlog
+//PG_STATE_SCANNING (1<<14) .. deprecated.
+#define PG_STATE_BACKFILL     (1<<15) // [active] backfilling pg content
+#define PG_STATE_INCOMPLETE   (1<<16) // incomplete content, peering failed.
 
 std::string pg_state_string(int state);
 
@@ -671,7 +673,6 @@ ostream& operator<<(ostream& out, const pg_pool_t& p);
  */
 struct object_stat_sum_t {
   int64_t num_bytes;    // in bytes
-  int64_t num_kb;       // in KB
   int64_t num_objects;
   int64_t num_object_clones;
   int64_t num_object_copies;  // num_objects * num_replicas
@@ -682,20 +683,22 @@ struct object_stat_sum_t {
   int64_t num_wr, num_wr_kb;
 
   object_stat_sum_t()
-    : num_bytes(0), num_kb(0),
+    : num_bytes(0),
       num_objects(0), num_object_clones(0), num_object_copies(0),
       num_objects_missing_on_primary(0), num_objects_degraded(0), num_objects_unfound(0),
       num_rd(0), num_rd_kb(0), num_wr(0), num_wr_kb(0)
   {}
 
-  void calc_copies_degraded(int nrep, int acting_nrep) {
+  void clear() {
+    memset(this, 0, sizeof(*this));
+  }
+
+  void calc_copies(int nrep) {
     num_object_copies = nrep * num_objects;
-    num_objects_degraded = (nrep - acting_nrep) * num_objects;
   }
 
   void dump(Formatter *f) const {
     f->dump_unsigned("num_bytes", num_bytes);
-    f->dump_unsigned("num_kb", num_kb);
     f->dump_unsigned("num_objects", num_objects);
     f->dump_unsigned("num_object_clones", num_object_clones);
     f->dump_unsigned("num_object_copies", num_object_copies);
@@ -711,6 +714,7 @@ struct object_stat_sum_t {
     __u8 v = 2;
     ::encode(v, bl);
     ::encode(num_bytes, bl);
+    uint64_t num_kb = SHIFT_ROUND_UP(num_bytes, 10);
     ::encode(num_kb, bl);
     ::encode(num_objects, bl);
     ::encode(num_object_clones, bl);
@@ -727,6 +731,7 @@ struct object_stat_sum_t {
     __u8 v;
     ::decode(v, bl);
     ::decode(num_bytes, bl);
+    uint64_t num_kb;
     ::decode(num_kb, bl);
     ::decode(num_objects, bl);
     ::decode(num_object_clones, bl);
@@ -742,7 +747,6 @@ struct object_stat_sum_t {
   }
   void add(const object_stat_sum_t& o) {
     num_bytes += o.num_bytes;
-    num_kb += o.num_kb;
     num_objects += o.num_objects;
     num_object_clones += o.num_object_clones;
     num_object_copies += o.num_object_copies;
@@ -756,7 +760,6 @@ struct object_stat_sum_t {
   }
   void sub(const object_stat_sum_t& o) {
     num_bytes -= o.num_bytes;
-    num_kb -= o.num_kb;
     num_objects -= o.num_objects;
     num_object_clones -= o.num_object_clones;
     num_object_copies -= o.num_object_copies;
@@ -775,10 +778,10 @@ struct object_stat_collection_t {
   object_stat_sum_t sum;
   map<string,object_stat_sum_t> cat_sum;
 
-  void calc_copies_degraded(int nrep, int acting_nrep) {
-    sum.calc_copies_degraded(nrep, acting_nrep);
+  void calc_copies(int nrep) {
+    sum.calc_copies(nrep);
     for (map<string,object_stat_sum_t>::iterator p = cat_sum.begin(); p != cat_sum.end(); ++p)
-      p->second.calc_copies_degraded(nrep, acting_nrep);
+      p->second.calc_copies(nrep);
   }
 
   void dump(Formatter *f) const {
@@ -806,6 +809,11 @@ struct object_stat_collection_t {
     ::decode(cat_sum, bl);
   }
 
+  void clear() {
+    sum.clear();
+    cat_sum.clear();
+  }
+
   void add(const object_stat_sum_t& o, const string& cat) {
     sum.add(o);
     if (cat.length())
@@ -935,7 +943,8 @@ struct pg_stat_t {
     ::decode(last_scrub_stamp, bl);
     if (v <= 4) {
       ::decode(stats.sum.num_bytes, bl);
-      ::decode(stats.sum.num_kb, bl);
+      uint64_t num_kb;
+      ::decode(num_kb, bl);
       ::decode(stats.sum.num_objects, bl);
       ::decode(stats.sum.num_object_clones, bl);
       ::decode(stats.sum.num_object_copies, bl);
@@ -1010,7 +1019,8 @@ struct pool_stat_t {
       ::decode(ondisk_log_size, bl);
     } else {
       ::decode(stats.sum.num_bytes, bl);
-      ::decode(stats.sum.num_kb, bl);
+      uint64_t num_kb;
+      ::decode(num_kb, bl);
       ::decode(stats.sum.num_objects, bl);
       ::decode(stats.sum.num_object_clones, bl);
       ::decode(stats.sum.num_object_copies, bl);
@@ -1302,12 +1312,49 @@ WRITE_CLASS_ENCODER(ScrubMap)
 
 struct OSDOp {
   ceph_osd_op op;
-  bufferlist data;
   sobject_t soid;
 
-  OSDOp() {
+  bufferlist indata, outdata;
+  int32_t rval;
+
+  OSDOp() : rval(0) {
     memset(&op, 0, sizeof(ceph_osd_op));
   }
+
+  /**
+   * split a bufferlist into constituent indata nembers of a vector of OSDOps
+   *
+   * @param ops [out] vector of OSDOps
+   * @param in  [in] combined data buffer
+   */
+  static void split_osd_op_vector_in_data(vector<OSDOp>& ops, bufferlist& in);
+
+  /**
+   * merge indata nembers of a vector of OSDOp into a single bufferlist
+   *
+   * Notably this also encodes certain other OSDOp data into the data
+   * buffer, including the sobject_t soid.
+   *
+   * @param ops [in] vector of OSDOps
+   * @param in  [out] combined data buffer
+   */
+  static void merge_osd_op_vector_in_data(vector<OSDOp>& ops, bufferlist& out);
+
+  /**
+   * split a bufferlist into constituent outdata members of a vector of OSDOps
+   *
+   * @param ops [out] vector of OSDOps
+   * @param in  [in] combined data buffer
+   */
+  static void split_osd_op_vector_out_data(vector<OSDOp>& ops, bufferlist& in);
+
+  /**
+   * merge outdata members of a vector of OSDOps into a single bufferlist
+   *
+   * @param ops [in] vector of OSDOps
+   * @param in  [out] combined data buffer
+   */
+  static void merge_osd_op_vector_out_data(vector<OSDOp>& ops, bufferlist& out);
 };
 
 ostream& operator<<(ostream& out, const OSDOp& op);
