@@ -78,6 +78,7 @@ void usage()
        << "  snap create <--snap=name> [image-name]    create a snapshot\n"
        << "  snap rollback <--snap=name> [image-name]  rollback image head to snapshot\n"
        << "  snap rm <--snap=name> [image-name]        deletes a snapshot\n"
+       << "  snap purge [image-name]                   deletes all snapshots\n"
        << "  watch [image-name]                        watch events on image\n"
        << "  map [image-name]                          map the image to a block device\n"
        << "                                            using the kernel\n"
@@ -99,7 +100,7 @@ void usage()
        << "\n"
        << "For the map command:\n"
        << "  --user <username>            rados user to authenticate as\n"
-       << "  --secret <path>              file containing secret key for use with authx\n";
+       << "  --secret <path>              file containing secret key for use with cephx\n";
 }
 
 void usage_exit()
@@ -259,6 +260,25 @@ static int do_rollback_snap(librbd::Image& image, const char *snapname)
   return 0;
 }
 
+static int do_purge_snaps(librbd::Image& image)
+{
+  MyProgressContext pc("Removing all snapshots");
+  std::vector<librbd::snap_info_t> snaps;
+  int r = image.snap_list(snaps);
+  if (r < 0) {
+    pc.fail();
+    return r;
+  }
+
+  for (size_t i = 0; i < snaps.size(); ++i) {
+    image.snap_remove(snaps[i].name.c_str());
+    pc.update_progress(i + 1, snaps.size());
+  }
+
+  pc.finish();
+  return 0;
+}
+
 struct ExportContext {
   int fd;
   MyProgressContext pc;
@@ -567,7 +587,9 @@ static int do_watch(librados::IoCtx& pp, const char *imgname)
   return 0;
 }
 
-static int do_kernel_add(const char *poolname, const char *imgname, const char *secretfile, const char *user)
+static int do_kernel_add(const char *poolname, const char *imgname,
+			 const char *snapname, const char *secretfile,
+			 const char *user)
 {
   MonMap monmap;
   int r = MonClient::build_initial_monmap(g_ceph_context, monmap);
@@ -584,24 +606,32 @@ static int do_kernel_add(const char *poolname, const char *imgname, const char *
 
   oss << " name=" << user;
 
+  char key_name[strlen(user) + strlen("client.")];
+  snprintf(key_name, sizeof(key_name), "client.%s", user);
+  char secret_buf[MAX_SECRET_LEN];
+  char *secret = NULL;
   if (secretfile) {
-    char secret[MAX_SECRET_LEN];
-    r = read_secret_from_file(secretfile, secret, sizeof(secret));
+    r = read_secret_from_file(secretfile, secret_buf, sizeof(secret_buf));
     if (r < 0)
       return r;
+    secret = secret_buf;
+  }
 
+  if (secret || is_kernel_secret(key_name)) {
     char option[MAX_SECRET_LEN + 7];
-    char key_name[strlen(user) + strlen("client.")];
-    snprintf(key_name, sizeof(key_name), "client.%s", user);
     r = get_secret_option(secret, key_name, option, sizeof(option));
-    if (r < 0)
+    if (r < 0) {
       return r;
-
+    }
     oss << "," << option;
   }
 
   oss << " " << poolname << " " << imgname;
 
+  if (snapname) {
+    oss << " " << snapname;
+  }
+
   // write to /sys/bus/rbd/add
   int fd = open("/sys/bus/rbd/add", O_WRONLY);
   if (fd < 0) {
@@ -805,6 +835,7 @@ enum {
   OPT_SNAP_ROLLBACK,
   OPT_SNAP_REMOVE,
   OPT_SNAP_LIST,
+  OPT_SNAP_PURGE,
   OPT_WATCH,
   OPT_MAP,
   OPT_UNMAP,
@@ -856,6 +887,8 @@ static int get_cmd(const char *cmd, bool snapcmd)
     if (strcmp(cmd, "ls") == 0||
         strcmp(cmd, "list") == 0)
       return OPT_SNAP_LIST;
+    if (strcmp(cmd, "purge") == 0)
+      return OPT_SNAP_PURGE;
   }
 
   return OPT_NO_CMD;
@@ -969,6 +1002,7 @@ int main(int argc, const char **argv)
       case OPT_SNAP_ROLLBACK:
       case OPT_SNAP_REMOVE:
       case OPT_SNAP_LIST:
+      case OPT_SNAP_PURGE:
       case OPT_WATCH:
       case OPT_MAP:
 	set_conf_param(v, &imgname, NULL);
@@ -1022,7 +1056,8 @@ int main(int argc, const char **argv)
 		      (char **)&imgname, (char **)&snapname);
   if (snapname && opt_cmd != OPT_SNAP_CREATE && opt_cmd != OPT_SNAP_ROLLBACK &&
       opt_cmd != OPT_SNAP_REMOVE && opt_cmd != OPT_INFO &&
-      opt_cmd != OPT_EXPORT && opt_cmd != OPT_COPY) {
+      opt_cmd != OPT_EXPORT && opt_cmd != OPT_COPY &&
+      opt_cmd != OPT_MAP) {
     cerr << "error: snapname specified for a command that doesn't use it" << std::endl;
     usage_exit();
   }
@@ -1047,7 +1082,9 @@ int main(int argc, const char **argv)
     usage_exit();
   }
 
-  bool talk_to_cluster = (opt_cmd != OPT_MAP && opt_cmd != OPT_UNMAP);
+  bool talk_to_cluster = (opt_cmd != OPT_MAP &&
+			  opt_cmd != OPT_UNMAP &&
+			  opt_cmd != OPT_SHOWMAPPED);
   if (talk_to_cluster && rados.init_with_context(g_ceph_context) < 0) {
     cerr << "error: couldn't initialize rados!" << std::endl;
     exit(1);
@@ -1070,8 +1107,8 @@ int main(int argc, const char **argv)
   if (imgname && talk_to_cluster &&
       (opt_cmd == OPT_RESIZE || opt_cmd == OPT_INFO || opt_cmd == OPT_SNAP_LIST ||
        opt_cmd == OPT_SNAP_CREATE || opt_cmd == OPT_SNAP_ROLLBACK ||
-       opt_cmd == OPT_SNAP_REMOVE || opt_cmd == OPT_EXPORT || opt_cmd == OPT_WATCH ||
-       opt_cmd == OPT_COPY)) {
+       opt_cmd == OPT_SNAP_REMOVE || opt_cmd == OPT_SNAP_PURGE ||
+       opt_cmd == OPT_EXPORT || opt_cmd == OPT_WATCH || opt_cmd == OPT_COPY)) {
     r = rbd.open(io_ctx, image, imgname);
     if (r < 0) {
       cerr << "error opening image " << imgname << ": " << cpp_strerror(-r) << std::endl;
@@ -1148,7 +1185,13 @@ int main(int argc, const char **argv)
   case OPT_RM:
     r = do_delete(rbd, io_ctx, imgname);
     if (r < 0) {
-      cerr << "delete error: " << cpp_strerror(-r) << std::endl;
+      if (r == -EBUSY) {
+	cerr << "delete error: image has snapshots - these must be deleted"
+	     << " with 'rbd snap purge' before the image can be removed."
+	     << std::endl;
+      } else {
+	cerr << "delete error: " << cpp_strerror(-r) << std::endl;
+      }
       exit(1);
     }
     break;
@@ -1209,6 +1252,18 @@ int main(int argc, const char **argv)
     }
     break;
 
+  case OPT_SNAP_PURGE:
+    if (!imgname) {
+      usage();
+      exit(1);
+    }
+    r = do_purge_snaps(image);
+    if (r < 0) {
+      cerr << "removing snaps failed: " << cpp_strerror(-r) << std::endl;
+      exit(1);
+    }
+    break;
+
   case OPT_EXPORT:
     if (!path) {
       cerr << "pathname should be specified" << std::endl;
@@ -1250,7 +1305,7 @@ int main(int argc, const char **argv)
     break;
 
   case OPT_MAP:
-    r = do_kernel_add(poolname, imgname, secretfile, user);
+    r = do_kernel_add(poolname, imgname, snapname, secretfile, user);
     if (r < 0) {
       cerr << "add failed: " << cpp_strerror(-r) << std::endl;
       exit(1);
