@@ -67,24 +67,26 @@ public:
                             bool system_bucket, bool exclusive = true,
                             uint64_t auid = 0) = 0;
   virtual int add_bucket_placement(std::string& new_placement) { return 0; }
+  virtual int remove_bucket_placement(std::string& new_placement) { return 0; }
+  virtual int list_placement_set(set<string>& names) { return 0; }
   virtual int create_pools(vector<string>& names, vector<int>& retcodes, int auid = 0) { return -ENOTSUP; }
   /** write an object to the storage device in the appropriate pool
     with the given stats */
   virtual int put_obj_meta(void *ctx, rgw_obj& obj, uint64_t size, time_t *mtime,
                       map<std::string, bufferlist>& attrs, RGWObjCategory category, bool exclusive,
-                      map<std::string, bufferlist>* rmattrs) = 0;
+                      map<std::string, bufferlist>* rmattrs,
+                      const bufferlist *data) = 0;
   virtual int put_obj_data(void *ctx, rgw_obj& obj, const char *data,
-                      off_t ofs, size_t len) = 0;
-  virtual int aio_put_obj_data(void *ctx, rgw_obj& obj, const char *data,
-                      off_t ofs, size_t len, void **handle) { return -ENOTSUP; }
+                      off_t ofs, size_t len, bool exclusive) = 0;
+  virtual int aio_put_obj_data(void *ctx, rgw_obj& obj, bufferlist& bl,
+                      off_t ofs, bool exclusive, void **handle) { return -ENOTSUP; }
 
   /* note that put_obj doesn't set category on an object, only use it for none user objects */
-  int put_obj(void *ctx, rgw_obj& obj, const char *data, size_t len,
+  int put_obj(void *ctx, rgw_obj& obj, const char *data, size_t len, bool exclusive,
               time_t *mtime, map<std::string, bufferlist>& attrs) {
-    int ret = put_obj_data(ctx, obj, data, -1, len);
-    if (ret >= 0) {
-      ret = put_obj_meta(ctx, obj, len, mtime, attrs, RGW_OBJ_CATEGORY_NONE, false, NULL);
-    }
+    bufferlist bl;
+    bl.append(data, len);
+    int ret = put_obj_meta(ctx, obj, len, mtime, attrs, RGW_OBJ_CATEGORY_NONE, exclusive, NULL, &bl);
     return ret;
   }
 
@@ -229,7 +231,7 @@ public:
  /**
   * stat an object
   */
-  virtual int obj_stat(void *ctx, rgw_obj& obj, uint64_t *psize, time_t *pmtime, map<string, bufferlist> *attrs) = 0;
+  virtual int obj_stat(void *ctx, rgw_obj& obj, uint64_t *psize, time_t *pmtime, map<string, bufferlist> *attrs, bufferlist *first_chunk) = 0;
 
   virtual bool supports_tmap() { return false; }
 
@@ -255,6 +257,7 @@ public:
   virtual void *create_context(void *user_ctx) { return NULL; }
   virtual void destroy_context(void *ctx) {}
   virtual void set_atomic(void *ctx, rgw_obj& obj) {}
+  virtual void set_prefetch_data(void *ctx, rgw_obj& obj) {}
 
   // to notify upper layer that we need to do some operation on an object, and it's up to
   // the upper layer to schedule this operation.. e.g., log intent in intent log
@@ -265,7 +268,7 @@ public:
   /* The bucket here can either be the bucket name identifier, or the ID
    * in period format: ".123" */
   virtual int get_bucket_info(void *ctx, string& bucket, RGWBucketInfo& info) = 0;
-  virtual int put_bucket_info(string& bucket_name, RGWBucketInfo& info) = 0;
+  virtual int put_bucket_info(string& bucket_name, RGWBucketInfo& info, bool exclusive) = 0;
 
 
   virtual int remove_temp_objects(string date, string time) {
