@@ -21,6 +21,8 @@ struct ObjectMetaInfo {
   uint64_t size;
   time_t mtime;
 
+  ObjectMetaInfo() : size(0), mtime(0) {}
+
   void encode(bufferlist& bl) const {
     __u8 struct_v = 1;
     ::encode(struct_v, bl);
@@ -76,6 +78,8 @@ struct RGWCacheNotifyInfo {
   off_t ofs;
   string ns;
 
+  RGWCacheNotifyInfo() : op(0), ofs(0) {}
+
   void encode(bufferlist& obl) const {
     __u8 struct_v = 1;
     ::encode(struct_v, obl);
@@ -173,14 +177,14 @@ public:
   int set_attr(void *ctx, rgw_obj& obj, const char *name, bufferlist& bl);
   int put_obj_meta(void *ctx, rgw_obj& obj, uint64_t size, time_t *mtime,
                    map<std::string, bufferlist>& attrs, RGWObjCategory category, bool exclusive,
-                   map<std::string, bufferlist>* rmattrs);
+                   map<std::string, bufferlist>* rmattrs, const bufferlist *data);
 
   int put_obj_data(void *ctx, rgw_obj& obj, const char *data,
-              off_t ofs, size_t len);
+              off_t ofs, size_t len, bool exclusive);
 
   int get_obj(void *ctx, void **handle, rgw_obj& obj, char **data, off_t ofs, off_t end);
 
-  int obj_stat(void *ctx, rgw_obj& obj, uint64_t *psize, time_t *pmtime, map<string, bufferlist> *attrs);
+  int obj_stat(void *ctx, rgw_obj& obj, uint64_t *psize, time_t *pmtime, map<string, bufferlist> *attrs, bufferlist *first_chunk);
 
   int delete_obj(void *ctx, rgw_obj& obj, bool sync);
 };
@@ -279,7 +283,7 @@ int RGWCache<T>::set_attr(void *ctx, rgw_obj& obj, const char *attr_name, buffer
 template <class T>
 int RGWCache<T>::put_obj_meta(void *ctx, rgw_obj& obj, uint64_t size, time_t *mtime,
                               map<std::string, bufferlist>& attrs, RGWObjCategory category, bool exclusive,
-                              map<std::string, bufferlist>* rmattrs)
+                              map<std::string, bufferlist>* rmattrs, const bufferlist *data)
 {
   rgw_bucket bucket;
   string oid;
@@ -291,8 +295,12 @@ int RGWCache<T>::put_obj_meta(void *ctx, rgw_obj& obj, uint64_t size, time_t *mt
     info.xattrs = attrs;
     info.status = 0;
     info.flags = CACHE_FLAG_XATTRS;
+    if (data) {
+      info.data = *data;
+      info.flags |= CACHE_FLAG_DATA;
+    }
   }
-  int ret = T::put_obj_meta(ctx, obj, size, mtime, attrs, category, exclusive, rmattrs);
+  int ret = T::put_obj_meta(ctx, obj, size, mtime, attrs, category, exclusive, rmattrs, data);
   if (cacheable) {
     string name = normal_name(bucket, oid);
     if (ret >= 0) {
@@ -310,7 +318,7 @@ int RGWCache<T>::put_obj_meta(void *ctx, rgw_obj& obj, uint64_t size, time_t *mt
 
 template <class T>
 int RGWCache<T>::put_obj_data(void *ctx, rgw_obj& obj, const char *data,
-              off_t ofs, size_t len)
+              off_t ofs, size_t len, bool exclusive)
 {
   rgw_bucket bucket;
   string oid;
@@ -327,7 +335,7 @@ int RGWCache<T>::put_obj_data(void *ctx, rgw_obj& obj, const char *data,
     info.status = 0;
     info.flags = CACHE_FLAG_DATA;
   }
-  int ret = T::put_obj_data(ctx, obj, data, ofs, len);
+  int ret = T::put_obj_data(ctx, obj, data, ofs, len, exclusive);
   if (cacheable) {
     string name = normal_name(bucket, oid);
     if (ret >= 0) {
@@ -344,13 +352,13 @@ int RGWCache<T>::put_obj_data(void *ctx, rgw_obj& obj, const char *data,
 }
 
 template <class T>
-int RGWCache<T>::obj_stat(void *ctx, rgw_obj& obj, uint64_t *psize, time_t *pmtime, map<string, bufferlist> *attrs)
+int RGWCache<T>::obj_stat(void *ctx, rgw_obj& obj, uint64_t *psize, time_t *pmtime, map<string, bufferlist> *attrs, bufferlist *first_chunk)
 {
   rgw_bucket bucket;
   string oid;
   normalize_bucket_and_obj(obj.bucket, obj.object, bucket, oid);
   if (bucket.name[0] != '.')
-    return T::obj_stat(ctx, obj, psize, pmtime, attrs);
+    return T::obj_stat(ctx, obj, psize, pmtime, attrs, first_chunk);
 
   string name = normal_name(bucket, oid);
 
@@ -367,7 +375,7 @@ int RGWCache<T>::obj_stat(void *ctx, rgw_obj& obj, uint64_t *psize, time_t *pmti
     mtime = info.meta.mtime;
     goto done;
   }
-  r = T::obj_stat(ctx, obj, &size, &mtime, &info.xattrs);
+  r = T::obj_stat(ctx, obj, &size, &mtime, &info.xattrs, first_chunk);
   if (r < 0) {
     if (r == -ENOENT) {
       info.status = r;
