@@ -173,7 +173,7 @@ static int get_policy_from_attr(void *ctx, RGWAccessControlPolicy *policy, rgw_o
       try {
         policy->decode(iter);
       } catch (buffer::error& err) {
-        dout(0) << "error: could not decode policy, caught buffer::error" << dendl;
+        dout(0) << "ERROR: could not decode policy, caught buffer::error" << dendl;
         return -EIO;
       }
       if (g_conf->debug_rgw >= 15) {
@@ -187,11 +187,11 @@ static int get_policy_from_attr(void *ctx, RGWAccessControlPolicy *policy, rgw_o
   return ret;
 }
 
-static int get_obj_attrs(struct req_state *s, rgw_obj& obj, map<string, bufferlist>& attrs)
+static int get_obj_attrs(struct req_state *s, rgw_obj& obj, map<string, bufferlist>& attrs, uint64_t *obj_size)
 {
   void *handle;
   int ret = rgwstore->prepare_get_obj(s->obj_ctx, obj, NULL, NULL, &attrs, NULL,
-                                      NULL, NULL, NULL, NULL, NULL, NULL, &handle, &s->err);
+                                      NULL, NULL, NULL, NULL, NULL, obj_size, &handle, &s->err);
   rgwstore->finish_get_obj(&handle);
   return ret;
 }
@@ -204,7 +204,7 @@ static int read_acls(struct req_state *s, RGWBucketInfo& bucket_info, RGWAccessC
   rgw_obj obj;
 
   if (bucket_info.flags & BUCKET_SUSPENDED) {
-    dout(0) << "bucket " << bucket_info.bucket.name << " is suspended" << dendl;
+    dout(0) << "NOTICE: bucket " << bucket_info.bucket.name << " is suspended" << dendl;
     return -ERR_USER_SUSPENDED;
   }
 
@@ -214,7 +214,6 @@ static int read_acls(struct req_state *s, RGWBucketInfo& bucket_info, RGWAccessC
     obj.set_ns(mp_ns);
   }
   obj.init(bucket, oid, object);
-
   int ret = get_policy_from_attr(s->obj_ctx, policy, obj);
   if (ret == -ENOENT && object.size()) {
     /* object does not exist checking the bucket's ACL to make sure
@@ -243,7 +242,7 @@ static int read_acls(struct req_state *s, RGWBucketInfo& bucket_info, RGWAccessC
  * only_bucket: If true, reads the bucket ACL rather than the object ACL.
  * Returns: 0 on success, -ERR# otherwise.
  */
-static int read_acls(struct req_state *s, bool only_bucket)
+static int read_acls(struct req_state *s, bool only_bucket, bool prefetch_data)
 {
   int ret = 0;
   string obj_str;
@@ -253,23 +252,27 @@ static int read_acls(struct req_state *s, bool only_bucket)
        return -ENOMEM;
   }
 
+
+  RGWBucketInfo bucket_info;
+  if (s->bucket_name_str.size()) {
+    ret = rgwstore->get_bucket_info(s->obj_ctx, s->bucket_name_str, bucket_info);
+    if (ret < 0) {
+      dout(0) << "NOTICE: couldn't get bucket from bucket_name (name=" << s->bucket_name_str << ")" << dendl;
+      return ret;
+    }
+    s->bucket = bucket_info.bucket;
+    s->bucket_owner = bucket_info.owner;
+  }
+
   /* we're passed only_bucket = true when we specifically need the bucket's
      acls, that happens on write operations */
   if (!only_bucket) {
     obj_str = s->object_str;
     rgw_obj obj(s->bucket, obj_str);
     rgwstore->set_atomic(s->obj_ctx, obj);
-  }
-
-  RGWBucketInfo bucket_info;
-  if (s->bucket_name_str.size()) {
-    ret = rgwstore->get_bucket_info(s->obj_ctx, s->bucket_name_str, bucket_info);
-    if (ret < 0) {
-      dout(0) << "couldn't get bucket from bucket_name (name=" << s->bucket_name_str << ")" << dendl;
-      return ret;
+    if (prefetch_data) {
+      rgwstore->set_prefetch_data(s->obj_ctx, obj);
     }
-    s->bucket = bucket_info.bucket;
-    s->bucket_owner = bucket_info.owner;
   }
 
   ret = read_acls(s, bucket_info, s->acl, s->bucket, obj_str);
@@ -279,6 +282,10 @@ static int read_acls(struct req_state *s, bool only_bucket)
 
 int RGWGetObj::verify_permission()
 {
+  obj.init(s->bucket, s->object_str);
+  rgwstore->set_atomic(s->obj_ctx, obj);
+  rgwstore->set_prefetch_data(s->obj_ctx, obj);
+
   if (!::verify_permission(s, RGW_PERM_READ))
     return -EACCES;
 
@@ -288,7 +295,6 @@ int RGWGetObj::verify_permission()
 void RGWGetObj::execute()
 {
   void *handle = NULL;
-  rgw_obj obj;
   utime_t start_time = s->time;
 
   perfcounter->inc(l_rgw_get);
@@ -301,8 +307,6 @@ void RGWGetObj::execute()
   if (ret < 0)
     goto done;
 
-  obj.init(s->bucket, s->object_str);
-  rgwstore->set_atomic(s->obj_ctx, obj);
   ret = rgwstore->prepare_get_obj(s->obj_ctx, obj, &ofs, &end, &attrs, mod_ptr,
                                   unmod_ptr, &lastmod, if_match, if_nomatch, &total_len, &s->obj_size, &handle, &s->err);
   if (ret < 0)
@@ -520,7 +524,6 @@ void RGWCreateBucket::execute()
   bool pol_ret;
 
   rgw_obj obj(rgw_root_bucket, s->bucket_name_str);
-
   s->bucket_owner = s->user.user_id;
 
   int r = get_policy_from_attr(s->obj_ctx, &old_policy, obj);
@@ -544,7 +547,7 @@ void RGWCreateBucket::execute()
                                 true, s->user.auid);
   /* continue if EEXIST and create_bucket will fail below.  this way we can recover
    * from a partial create by retrying it. */
-  dout(0) << "rgw_create_bucket returned ret=" << ret << " bucket=" << s->bucket << dendl;
+  dout(20) << "rgw_create_bucket returned ret=" << ret << " bucket=" << s->bucket << dendl;
 
   if (ret && ret != -EEXIST)   
     goto done;
@@ -597,46 +600,9 @@ void RGWDeleteBucket::execute()
 }
 
 struct put_obj_aio_info {
-  void *data;
   void *handle;
 };
 
-static struct put_obj_aio_info pop_pending(std::list<struct put_obj_aio_info>& pending)
-{
-  struct put_obj_aio_info info;
-  info = pending.front();
-  pending.pop_front();
-  return info;
-}
-
-static int wait_pending_front(std::list<struct put_obj_aio_info>& pending)
-{
-  struct put_obj_aio_info info = pop_pending(pending);
-  int ret = rgwstore->aio_wait(info.handle);
-  free(info.data);
-  return ret;
-}
-
-static bool pending_has_completed(std::list<struct put_obj_aio_info>& pending)
-{
-  if (pending.size() == 0)
-    return false;
-
-  struct put_obj_aio_info& info = pending.front();
-  return rgwstore->aio_completed(info.handle);
-}
-
-static int drain_pending(std::list<struct put_obj_aio_info>& pending)
-{
-  int ret = 0;
-  while (!pending.empty()) {
-    int r = wait_pending_front(pending);
-    if (r < 0)
-      ret = r;
-  }
-  return ret;
-}
-
 int RGWPutObj::verify_permission()
 {
   if (!::verify_permission(s, RGW_PERM_WRITE))
@@ -645,214 +611,397 @@ int RGWPutObj::verify_permission()
   return 0;
 }
 
-void RGWPutObj::execute()
+class RGWPutObjProcessor_Plain : public RGWPutObjProcessor
+{
+  bufferlist data;
+  rgw_obj obj;
+  off_t ofs;
+
+protected:
+  int prepare(struct req_state *s);
+  int handle_data(bufferlist& bl, off_t ofs, void **phandle);
+  int throttle_data(void *handle) { return 0; }
+  int complete(string& etag, map<string, bufferlist>& attrs);
+
+public:
+  RGWPutObjProcessor_Plain() : ofs(0) {}
+};
+
+int RGWPutObjProcessor_Plain::prepare(struct req_state *s)
+{
+  RGWPutObjProcessor::prepare(s);
+
+  obj.init(s->bucket, s->object_str);
+
+  return 0;
+};
+
+int RGWPutObjProcessor_Plain::handle_data(bufferlist& bl, off_t _ofs, void **phandle)
+{
+  if (ofs != _ofs)
+    return -EINVAL;
+
+  data.append(bl);
+  ofs += bl.length();
+
+  return 0;
+}
+
+int RGWPutObjProcessor_Plain::complete(string& etag, map<string, bufferlist>& attrs)
+{
+  int r = rgwstore->put_obj_meta(s->obj_ctx, obj, data.length(), NULL, attrs,
+                                 RGW_OBJ_CATEGORY_MAIN, false, NULL, &data);
+  return r;
+}
+
+
+class RGWPutObjProcessor_Aio : public RGWPutObjProcessor
 {
-  bool multipart;
-  string multipart_meta_obj;
-  string part_num;
   list<struct put_obj_aio_info> pending;
-  size_t max_chunks = RGW_MAX_PENDING_CHUNKS;
-  bool created_obj = false;
+  size_t max_chunks;
+
+  struct put_obj_aio_info pop_pending();
+  int wait_pending_front();
+  bool pending_has_completed();
+  int drain_pending();
+
+protected:
   rgw_obj obj;
 
-  perfcounter->inc(l_rgw_put);
+  int handle_data(bufferlist& bl, off_t ofs, void **phandle);
+  int throttle_data(void *handle);
 
+  RGWPutObjProcessor_Aio() : max_chunks(RGW_MAX_PENDING_CHUNKS) {}
+  virtual ~RGWPutObjProcessor_Aio() {
+    drain_pending();
+  }
+};
+
+int RGWPutObjProcessor_Aio::handle_data(bufferlist& bl, off_t ofs, void **phandle)
+{
+  // For the first call pass -1 as the offset to
+  // do a write_full.
+  int r = rgwstore->aio_put_obj_data(s->obj_ctx, obj,
+                                     bl,
+                                     ((ofs == 0) ? -1 : ofs),
+                                     false, phandle);
+
+  return r;
+}
+
+struct put_obj_aio_info RGWPutObjProcessor_Aio::pop_pending()
+{
+  struct put_obj_aio_info info;
+  info = pending.front();
+  pending.pop_front();
+  return info;
+}
+
+int RGWPutObjProcessor_Aio::wait_pending_front()
+{
+  struct put_obj_aio_info info = pop_pending();
+  int ret = rgwstore->aio_wait(info.handle);
+  return ret;
+}
+
+bool RGWPutObjProcessor_Aio::pending_has_completed()
+{
+  if (pending.size() == 0)
+    return false;
+
+  struct put_obj_aio_info& info = pending.front();
+  return rgwstore->aio_completed(info.handle);
+}
+
+int RGWPutObjProcessor_Aio::drain_pending()
+{
+  int ret = 0;
+  while (!pending.empty()) {
+    int r = wait_pending_front();
+    if (r < 0)
+      ret = r;
+  }
+  return ret;
+}
+
+int RGWPutObjProcessor_Aio::throttle_data(void *handle)
+{
+  struct put_obj_aio_info info;
+  info.handle = handle;
+  pending.push_back(info);
+  size_t orig_size = pending.size();
+  while (pending_has_completed()) {
+    int r = wait_pending_front();
+    if (r < 0)
+      return r;
+  }
+
+  /* resize window in case messages are draining too fast */
+  if (orig_size - pending.size() >= max_chunks)
+  max_chunks++;
+
+  if (pending.size() > max_chunks) {
+    int r = wait_pending_front();
+    if (r < 0)
+      return r;
+  }
+  return 0;
+}
+
+class RGWPutObjProcessor_Atomic : public RGWPutObjProcessor_Aio
+{
+  bool remove_temp_obj;
+protected:
+  int prepare(struct req_state *s);
+  int complete(string& etag, map<string, bufferlist>& attrs);
+
+public:
+  ~RGWPutObjProcessor_Atomic();
+  RGWPutObjProcessor_Atomic() : remove_temp_obj(false) {}
+  int handle_data(bufferlist& bl, off_t ofs, void **phandle) {
+    int r = RGWPutObjProcessor_Aio::handle_data(bl, ofs, phandle);
+    if (r >= 0) {
+      remove_temp_obj = true;
+    }
+    return r;
+  }
+};
+
+int RGWPutObjProcessor_Atomic::prepare(struct req_state *s)
+{
+  RGWPutObjProcessor::prepare(s);
+
+  string oid = s->object_str;
+  obj.set_ns(tmp_ns);
+
+  char buf[33];
+  gen_rand_alphanumeric(buf, sizeof(buf) - 1);
+  oid.append("_");
+  oid.append(buf);
+  obj.init(s->bucket, oid, s->object_str);
+
+  return 0;
+}
+
+int RGWPutObjProcessor_Atomic::complete(string& etag, map<string, bufferlist>& attrs)
+{
+  rgw_obj dst_obj(s->bucket, s->object_str);
+  rgwstore->set_atomic(s->obj_ctx, dst_obj);
+  int r = rgwstore->clone_obj(s->obj_ctx, dst_obj, 0, obj, 0, s->obj_size, NULL, attrs, RGW_OBJ_CATEGORY_MAIN);
+
+  return r;
+}
+RGWPutObjProcessor_Atomic::~RGWPutObjProcessor_Atomic()
+{
+  if (remove_temp_obj)
+    rgwstore->delete_obj(NULL, obj, NULL);
+}
+
+class RGWPutObjProcessor_Multipart : public RGWPutObjProcessor_Aio
+{
+  string part_num;
+  RGWMPObj mp;
+protected:
+  int prepare(struct req_state *s);
+  int complete(string& etag, map<string, bufferlist>& attrs);
+
+public:
+  RGWPutObjProcessor_Multipart() {}
+};
+
+int RGWPutObjProcessor_Multipart::prepare(struct req_state *s)
+{
+  RGWPutObjProcessor::prepare(s);
+
+  string oid = s->object_str;
+  string upload_id;
+  url_decode(s->args.get("uploadId"), upload_id);
+  mp.init(oid, upload_id);
+
+  url_decode(s->args.get("partNumber"), part_num);
+  if (part_num.empty()) {
+    return -EINVAL;
+  }
+  oid = mp.get_part(part_num);
+
+  obj.set_ns(mp_ns);
+  obj.init(s->bucket, oid, s->object_str);
+  return 0;
+}
+
+int RGWPutObjProcessor_Multipart::complete(string& etag, map<string, bufferlist>& attrs)
+{
+  int r = rgwstore->put_obj_meta(s->obj_ctx, obj, s->obj_size, NULL, attrs, RGW_OBJ_CATEGORY_MAIN, false, NULL, NULL);
+  if (r < 0)
+    return r;
+
+  bufferlist bl;
+  RGWUploadPartInfo info;
+  string p = "part.";
+  p.append(part_num);
+  info.num = atoi(part_num.c_str());
+  info.etag = etag;
+  info.size = s->obj_size;
+  info.modified = ceph_clock_now(g_ceph_context);
+  ::encode(info, bl);
+
+  string multipart_meta_obj = mp.get_meta();
+
+  rgw_obj meta_obj(s->bucket, multipart_meta_obj, s->object_str, mp_ns);
+
+  r = rgwstore->tmap_set(meta_obj, p, bl);
+
+  return r;
+}
+
+
+RGWPutObjProcessor *RGWPutObj::select_processor()
+{
+  RGWPutObjProcessor *processor;
+
+  bool multipart = s->args.exists("uploadId");
+
+  if (!multipart) {
+    if (s->content_length <= RGW_MAX_CHUNK_SIZE && !chunked_upload)
+      processor = new RGWPutObjProcessor_Plain();
+    else
+      processor = new RGWPutObjProcessor_Atomic();
+  } else {
+    processor = new RGWPutObjProcessor_Multipart();
+  }
+
+  return processor;
+}
+
+void RGWPutObj::dispose_processor(RGWPutObjProcessor *processor)
+{
+  delete processor;
+}
+
+void RGWPutObj::execute()
+{
+  rgw_obj obj;
+  RGWAccessControlPolicy policy;
+  RGWPutObjProcessor *processor = NULL;
+  char supplied_md5_bin[CEPH_CRYPTO_MD5_DIGESTSIZE + 1];
+  char supplied_md5[CEPH_CRYPTO_MD5_DIGESTSIZE * 2 + 1];
+  char calc_md5[CEPH_CRYPTO_MD5_DIGESTSIZE * 2 + 1];
+  unsigned char m[CEPH_CRYPTO_MD5_DIGESTSIZE];
+  MD5 hash;
+  bufferlist bl, aclbl;
+  map<string, bufferlist> attrs;
+  int len;
+
+
+  perfcounter->inc(l_rgw_put);
   ret = -EINVAL;
   if (!s->object) {
     goto done;
-  } else {
-    ret = get_params();
+  }
+
+  ret = get_params();
+  if (ret < 0)
+    goto done;
+
+  ret = policy.create_canned(s->user.user_id, s->user.display_name, s->canned_acl);
+  if (!ret) {
+     ret = -EINVAL;
+     goto done;
+  }
+
+  if (supplied_md5_b64) {
+    dout(15) << "supplied_md5_b64=" << supplied_md5_b64 << dendl;
+    ret = ceph_unarmor(supplied_md5_bin, &supplied_md5_bin[CEPH_CRYPTO_MD5_DIGESTSIZE + 1],
+                       supplied_md5_b64, supplied_md5_b64 + strlen(supplied_md5_b64));
+    dout(15) << "ceph_armor ret=" << ret << dendl;
+    if (ret != CEPH_CRYPTO_MD5_DIGESTSIZE) {
+      ret = -ERR_INVALID_DIGEST;
+      goto done;
+    }
+
+    buf_to_hex((const unsigned char *)supplied_md5_bin, CEPH_CRYPTO_MD5_DIGESTSIZE, supplied_md5);
+    dout(15) << "supplied_md5=" << supplied_md5 << dendl;
+  }
+
+  if (supplied_etag) {
+    strncpy(supplied_md5, supplied_etag, sizeof(supplied_md5));
+  }
+
+  processor = select_processor();
+
+  ret = processor->prepare(s);
+  if (ret < 0)
+    goto done;
+
+  do {
+    bufferlist data;
+    len = get_data(data);
+    if (len < 0) {
+      ret = len;
+      goto done;
+    }
+    if (!len)
+      break;
+
+    void *handle;
+    ret = processor->handle_data(data, ofs, &handle);
     if (ret < 0)
       goto done;
 
-    RGWAccessControlPolicy policy;
+    hash.Update((unsigned char *)data.c_str(), len);
 
-    ret = policy.create_canned(s->user.user_id, s->user.display_name, s->canned_acl);
-    if (!ret) {
-       ret = -EINVAL;
-       goto done;
-    }
-    char supplied_md5_bin[CEPH_CRYPTO_MD5_DIGESTSIZE + 1];
-    char supplied_md5[CEPH_CRYPTO_MD5_DIGESTSIZE * 2 + 1];
-    char calc_md5[CEPH_CRYPTO_MD5_DIGESTSIZE * 2 + 1];
-    unsigned char m[CEPH_CRYPTO_MD5_DIGESTSIZE];
+    ret = processor->throttle_data(handle);
+    if (ret < 0)
+      goto done;
 
-    if (supplied_md5_b64) {
-      dout(15) << "supplied_md5_b64=" << supplied_md5_b64 << dendl;
-      ret = ceph_unarmor(supplied_md5_bin, &supplied_md5_bin[CEPH_CRYPTO_MD5_DIGESTSIZE + 1],
-			     supplied_md5_b64, supplied_md5_b64 + strlen(supplied_md5_b64));
-      dout(15) << "ceph_armor ret=" << ret << dendl;
-      if (ret != CEPH_CRYPTO_MD5_DIGESTSIZE) {
-        ret = -ERR_INVALID_DIGEST;
-        goto done;
-      }
+    ofs += len;
+  } while (len > 0);
 
-      buf_to_hex((const unsigned char *)supplied_md5_bin, CEPH_CRYPTO_MD5_DIGESTSIZE, supplied_md5);
-      dout(15) << "supplied_md5=" << supplied_md5 << dendl;
-    }
+  // was this really needed? processor->complete() will synch
+  // drain_pending(pending);
 
-    if (supplied_etag) {
-      strncpy(supplied_md5, supplied_etag, sizeof(supplied_md5));
-    }
-
-    MD5 hash;
-    string oid;
-    multipart = s->args.exists("uploadId");
-    if (!multipart) {
-      oid = s->object_str;
-      obj.set_ns(tmp_ns);
-
-      char buf[33];
-      gen_rand_alphanumeric(buf, sizeof(buf) - 1);
-      oid.append("_");
-      oid.append(buf);
-    } else {
-      oid = s->object_str;
-      string upload_id;
-      url_decode(s->args.get("uploadId"), upload_id);
-      RGWMPObj mp(oid, upload_id);
-      multipart_meta_obj = mp.get_meta();
-
-      url_decode(s->args.get("partNumber"), part_num);
-      if (part_num.empty()) {
-        ret = -EINVAL;
-        goto done;
-      }
-      oid = mp.get_part(part_num);
-
-      obj.set_ns(mp_ns);
-    }
-    obj.init(s->bucket, oid, s->object_str);
-    int len;
-    do {
-      len = get_data();
-      if (len < 0) {
-        ret = len;
-        goto done;
-      }
-      if (len > 0) {
-        struct put_obj_aio_info info;
-        size_t orig_size;
-	// For the first call to put_obj_data, pass -1 as the offset to
-	// do a write_full.
-        void *handle;
-        ret = rgwstore->aio_put_obj_data(s->obj_ctx, obj,
-				     data,
-				     ((ofs == 0) ? -1 : ofs), len, &handle);
-        if (ret < 0)
-          goto done_err;
-
-        created_obj = true;
-
-        hash.Update((unsigned char *)data, len);
-        info.handle = handle;
-        info.data = data;
-        pending.push_back(info);
-        orig_size = pending.size();
-        while (pending_has_completed(pending)) {
-          ret = wait_pending_front(pending);
-          if (ret < 0)
-            goto done_err;
-
-        }
-
-        /* resize window in case messages are draining too fast */
-        if (orig_size - pending.size() >= max_chunks)
-          max_chunks++;
-
-        if (pending.size() > max_chunks) {
-          ret = wait_pending_front(pending);
-          if (ret < 0)
-            goto done_err;
-        }
-        ofs += len;
-      }
-    } while ( len > 0);
-    drain_pending(pending);
-
-    if (!chunked_upload && (uint64_t)ofs != s->content_length) {
-      ret = -ERR_REQUEST_TIMEOUT;
-      goto done_err;
-    }
-    s->obj_size = ofs;
-    perfcounter->inc(l_rgw_put_b, s->obj_size);
-
-    hash.Final(m);
-
-    buf_to_hex(m, CEPH_CRYPTO_MD5_DIGESTSIZE, calc_md5);
-
-    if (supplied_md5_b64 && strcmp(calc_md5, supplied_md5)) {
-       ret = -ERR_BAD_DIGEST;
-       goto done_err;
-    }
-    bufferlist aclbl;
-    policy.encode(aclbl);
-
-    etag = calc_md5;
-
-    if (supplied_etag && etag.compare(supplied_etag) != 0) {
-      ret = -ERR_UNPROCESSABLE_ENTITY;
-      goto done_err;
-    }
-    map<string, bufferlist> attrs;
-    bufferlist bl;
-    bl.append(etag.c_str(), etag.size() + 1);
-    attrs[RGW_ATTR_ETAG] = bl;
-    attrs[RGW_ATTR_ACL] = aclbl;
-
-    if (s->content_type) {
-      bl.clear();
-      bl.append(s->content_type, strlen(s->content_type) + 1);
-      attrs[RGW_ATTR_CONTENT_TYPE] = bl;
-    }
-
-    get_request_metadata(s, attrs);
-
-    if (!multipart) {
-      rgw_obj dst_obj(s->bucket, s->object_str);
-      rgwstore->set_atomic(s->obj_ctx, dst_obj);
-      ret = rgwstore->clone_obj(s->obj_ctx, dst_obj, 0, obj, 0, s->obj_size, NULL, attrs, RGW_OBJ_CATEGORY_MAIN);
-      if (ret < 0)
-        goto done_err;
-      if (created_obj) {
-        ret = rgwstore->delete_obj(NULL, obj, false);
-        if (ret < 0)
-          goto done;
-      }
-    } else {
-      ret = rgwstore->put_obj_meta(s->obj_ctx, obj, s->obj_size, NULL, attrs, RGW_OBJ_CATEGORY_MAIN, false, NULL);
-      if (ret < 0)
-        goto done_err;
-
-      bl.clear();
-      RGWUploadPartInfo info;
-      string p = "part.";
-      p.append(part_num);
-      info.num = atoi(part_num.c_str());
-      info.etag = etag;
-      info.size = s->obj_size;
-      info.modified = ceph_clock_now(g_ceph_context);
-      ::encode(info, bl);
-
-      rgw_obj meta_obj(s->bucket, multipart_meta_obj, s->object_str, mp_ns);
-
-      ret = rgwstore->tmap_set(meta_obj, p, bl);
-    }
+  if (!chunked_upload && (uint64_t)ofs != s->content_length) {
+    ret = -ERR_REQUEST_TIMEOUT;
+    goto done;
   }
+  s->obj_size = ofs;
+  perfcounter->inc(l_rgw_put_b, s->obj_size);
+
+  hash.Final(m);
+
+  buf_to_hex(m, CEPH_CRYPTO_MD5_DIGESTSIZE, calc_md5);
+
+  if (supplied_md5_b64 && strcmp(calc_md5, supplied_md5)) {
+     ret = -ERR_BAD_DIGEST;
+     goto done;
+  }
+  policy.encode(aclbl);
+
+  etag = calc_md5;
+
+  if (supplied_etag && etag.compare(supplied_etag) != 0) {
+    ret = -ERR_UNPROCESSABLE_ENTITY;
+    goto done;
+  }
+  bl.append(etag.c_str(), etag.size() + 1);
+  attrs[RGW_ATTR_ETAG] = bl;
+  attrs[RGW_ATTR_ACL] = aclbl;
+
+  if (s->content_type) {
+    bl.clear();
+    bl.append(s->content_type, strlen(s->content_type) + 1);
+    attrs[RGW_ATTR_CONTENT_TYPE] = bl;
+  }
+
+  get_request_metadata(s, attrs);
+
+  ret = processor->complete(etag, attrs);
 done:
-  drain_pending(pending);
+  dispose_processor(processor);
   perfcounter->finc(l_rgw_put_lat,
                    (ceph_clock_now(g_ceph_context) - s->time));
   send_response();
   return;
-
-done_err:
-  if (created_obj)
-    rgwstore->delete_obj(s->obj_ctx, obj);
-  drain_pending(pending);
-  perfcounter->finc(l_rgw_put_lat,
-                   (ceph_clock_now(g_ceph_context) - s->time));
-  send_response();
 }
 
 int RGWPutObjMetadata::verify_permission()
@@ -877,8 +1026,10 @@ void RGWPutObjMetadata::execute()
 
   rgwstore->set_atomic(s->obj_ctx, obj);
 
+  uint64_t obj_size;
+
   /* check if obj exists, read orig attrs */
-  ret = get_obj_attrs(s, obj, orig_attrs);
+  ret = get_obj_attrs(s, obj, orig_attrs, &obj_size);
   if (ret < 0)
     goto done;
 
@@ -887,10 +1038,12 @@ void RGWPutObjMetadata::execute()
     const string& name = iter->first;
     if (name.compare(0, meta_prefix_len, meta_prefix) == 0) {
       rmattrs[name] = iter->second;
+    } else if (attrs.find(name) == attrs.end()) {
+      attrs[name] = iter->second;
     }
   }
 
-  ret = rgwstore->put_obj_meta(s->obj_ctx, obj, s->obj_size, NULL, attrs, RGW_OBJ_CATEGORY_MAIN, false, &rmattrs);
+  ret = rgwstore->put_obj_meta(s->obj_ctx, obj, obj_size, NULL, attrs, RGW_OBJ_CATEGORY_MAIN, false, &rmattrs, NULL);
 
 done:
   send_response();
@@ -1062,7 +1215,7 @@ int RGWGetACLs::verify_permission()
 
 void RGWGetACLs::execute()
 {
-  ret = read_acls(s, false);
+  ret = read_acls(s, false, false);
 
   if (ret < 0) {
     send_response();
@@ -1311,7 +1464,7 @@ void RGWInitMultipart::execute()
 
     obj.init(s->bucket, tmp_obj_name, s->object_str, mp_ns);
     // the meta object will be indexed with 0 size, we c
-    ret = rgwstore->put_obj_meta(s->obj_ctx, obj, 0, NULL, attrs, RGW_OBJ_CATEGORY_MULTIMETA, true, NULL);
+    ret = rgwstore->put_obj_meta(s->obj_ctx, obj, 0, NULL, attrs, RGW_OBJ_CATEGORY_MULTIMETA, true, NULL, NULL);
   } while (ret == -EEXIST);
 done:
   send_response();
@@ -1326,7 +1479,7 @@ static int get_multiparts_info(struct req_state *s, string& meta_oid, map<uint32
 
   rgw_obj obj(s->bucket, meta_oid, s->object_str, mp_ns);
 
-  int ret = get_obj_attrs(s, obj, attrs);
+  int ret = get_obj_attrs(s, obj, attrs, NULL);
   if (ret < 0)
     return ret;
 
@@ -1434,12 +1587,12 @@ void RGWCompleteMultipart::execute()
        ++iter, ++obj_iter) {
     char etag[CEPH_CRYPTO_MD5_DIGESTSIZE];
     if (iter->first != (int)obj_iter->first) {
-      dout(0) << "parts num mismatch: next requested: " << iter->first << " next uploaded: " << obj_iter->first << dendl;
+      dout(0) << "NOTICE: parts num mismatch: next requested: " << iter->first << " next uploaded: " << obj_iter->first << dendl;
       ret = -ERR_INVALID_PART;
       goto done;
     }
     if (iter->second.compare(obj_iter->second.etag) != 0) {
-      dout(0) << "etag mismatch: part: " << iter->first << " etag: " << iter->second << dendl;
+      dout(0) << "NOTICE: etag mismatch: part: " << iter->first << " etag: " << iter->second << dendl;
       ret = -ERR_INVALID_PART;
       goto done;
     }
@@ -1460,7 +1613,7 @@ void RGWCompleteMultipart::execute()
 
   target_obj.init(s->bucket, s->object_str);
   rgwstore->set_atomic(s->obj_ctx, target_obj);
-  ret = rgwstore->put_obj_meta(s->obj_ctx, target_obj, 0, NULL, attrs, RGW_OBJ_CATEGORY_MAIN, false, NULL);
+  ret = rgwstore->put_obj_meta(s->obj_ctx, target_obj, 0, NULL, attrs, RGW_OBJ_CATEGORY_MAIN, false, NULL, NULL);
   if (ret < 0)
     goto done;
   
@@ -1633,9 +1786,9 @@ int RGWHandler::init(struct req_state *_s, FCGX_Request *fcgx)
   return 0;
 }
 
-int RGWHandler::do_read_permissions(bool only_bucket)
+int RGWHandler::do_read_permissions(RGWOp *op, bool only_bucket)
 {
-  int ret = read_acls(s, only_bucket);
+  int ret = read_acls(s, only_bucket, op->prefetch_data());
 
   if (ret < 0) {
     dout(10) << "read_permissions on " << s->bucket << ":" <<s->object_str << " only_bucket=" << only_bucket << " ret=" << ret << dendl;
