@@ -281,20 +281,22 @@ int RGWRados::log_show_init(const string& name, RGWAccessHandle *handle)
 int RGWRados::log_show_next(RGWAccessHandle handle, rgw_log_entry *entry)
 {
   log_show_state *state = (log_show_state *)handle;
+  off_t off = state->p.get_off();
 
   dout(10) << "log_show_next pos " << state->pos << " bl " << state->bl.length()
-	   << " off " << state->p.get_off()
+	   << " off " << off
 	   << " eof " << (int)state->eof
 	   << dendl;
   // read some?
   unsigned chunk = 1024*1024;
-  if (state->bl.length() < chunk/2 && !state->eof) {
+  if ((state->bl.length() - off) < chunk/2 && !state->eof) {
     bufferlist more;
     int r = state->io_ctx.read(state->name, more, chunk, state->pos);
     if (r < 0)
       return r;
+    state->pos += r;
     bufferlist old;
-    old.substr_of(state->bl, state->p.get_off(), state->bl.length() - state->p.get_off());
+    old.substr_of(state->bl, off, state->bl.length() - off);
     state->bl.clear();
     state->bl.claim(old);
     state->bl.claim_append(more);
@@ -469,7 +471,9 @@ int RGWRados::create_bucket(string& owner, rgw_bucket& bucket,
     string dir_oid =  dir_oid_prefix;
     dir_oid.append(bucket.marker);
 
-    r = io_ctx.create(dir_oid, true);
+    librados::ObjectWriteOperation op;
+    op.create(true);
+    r = cls_rgw_init_index(io_ctx, op, dir_oid);
     if (r < 0 && r != -EEXIST)
       return r;
 
@@ -479,12 +483,6 @@ int RGWRados::create_bucket(string& owner, rgw_bucket& bucket,
     ret = store_bucket_info(info, &attrs, exclusive);
     if (ret == -EEXIST)
       return ret;
-
-    if (r != -EEXIST) {
-      r = cls_rgw_init_index(bucket, dir_oid);
-      if (r < 0)
-        return r;
-    }
   }
 
   return ret;
@@ -1696,9 +1694,9 @@ int RGWRados::get_obj(void *ctx, void **handle, rgw_obj& obj,
   }
 
   dout(20) << "rados->read ofs=" << ofs << " len=" << len << dendl;
-  op.read(ofs, len);
+  op.read(ofs, len, &bl, NULL);
 
-  r = state->io_ctx.operate(oid, &op, &bl);
+  r = state->io_ctx.operate(oid, &op, NULL);
   dout(20) << "rados->read r=" << r << " bl.length=" << bl.length() << dendl;
 
   if (r == -ECANCELED) {
@@ -1757,9 +1755,9 @@ int RGWRados::read(void *ctx, rgw_obj& obj, off_t ofs, size_t size, bufferlist&
   if (r < 0)
     return r;
 
-  op.read(ofs, size);
+  op.read(ofs, size, &bl, NULL);
 
-  r = io_ctx.operate(oid, &op, &bl);
+  r = io_ctx.operate(oid, &op, NULL);
   if (r == -ECANCELED) {
     /* a race! object was replaced, we need to set attr on the original obj */
     dout(0) << "NOTICE: RGWRados::get_obj: raced with another process, going to the shadow obj instead" << dendl;
@@ -1782,45 +1780,21 @@ int RGWRados::obj_stat(void *ctx, rgw_obj& obj, uint64_t *psize, time_t *pmtime,
 
   io_ctx.locator_set_key(key);
 
+  map<string, bufferlist> attrset;
+  uint64_t size = 0;
+  time_t mtime = 0;
+
   ObjectReadOperation op;
-  op.getxattrs();
-  op.stat();
+  op.getxattrs(&attrset, NULL);
+  op.stat(&size, &mtime, NULL);
   if (first_chunk) {
-    op.read(0, RGW_MAX_CHUNK_SIZE);
+    op.read(0, RGW_MAX_CHUNK_SIZE, first_chunk, NULL);
   }
   bufferlist outbl;
   r = io_ctx.operate(oid, &op, &outbl);
   if (r < 0)
     return r;
 
-  map<string, bufferlist> attrset;
-  bufferlist::iterator oiter = outbl.begin();
-  try {
-    ::decode(attrset, oiter);
-  } catch (buffer::error& err) {
-    dout(0) << "ERROR: failed decoding s->attrset (obj=" << obj << "), aborting" << dendl;
-    return -EIO;
-  }
-
-  map<string, bufferlist>::iterator aiter;
-  for (aiter = attrset.begin(); aiter != attrset.end(); ++aiter) {
-    dout(20) << "RGWRados::obj_stat: attr=" << aiter->first << dendl;
-  }
-
-  uint64_t size = 0;
-  time_t mtime = 0;
-  try {
-    ::decode(size, oiter);
-    utime_t ut;
-    ::decode(ut, oiter);
-    mtime = ut.sec();
-  } catch (buffer::error& err) {
-    dout(0) << "ERROR: failed decoding object (obj=" << obj << ") info (either size or mtime), aborting" << dendl;
-    return -EIO;
-  }
-  if (first_chunk) {
-    oiter.copy_all(*first_chunk);
-  }
   if (psize)
     *psize = size;
   if (pmtime)
@@ -2115,23 +2089,11 @@ int RGWRados::pool_list(rgw_bucket& bucket, string start, uint32_t num, map<stri
   return m.size();
 }
 
-int RGWRados::cls_rgw_init_index(rgw_bucket& bucket, string& oid)
+int RGWRados::cls_rgw_init_index(librados::IoCtx& io_ctx, librados::ObjectWriteOperation& op, string& oid)
 {
-  if (bucket_is_system(bucket))
-    return 0;
-
-  if (bucket.marker.empty()) {
-    dout(0) << "ERROR: empty marker for cls_rgw bucket operation" << dendl;
-    return -EIO;
-  }
-
-  librados::IoCtx io_ctx;
-  int r = open_bucket_ctx(bucket, io_ctx);
-  if (r < 0)
-    return r;
-
-  bufferlist in, out;
-  r = io_ctx.exec(oid, "rgw", "bucket_init_index", in, out);
+  bufferlist in;
+  op.exec("rgw", "bucket_init_index", in);
+  int r = io_ctx.operate(oid, &op);
   return r;
 }
 
@@ -2425,6 +2387,7 @@ int RGWRados::remove_temp_objects(string date, string time)
     format.append(" %H:%M:%S");
     datetime.append(time.c_str());
   }
+  memset(&tm, 0, sizeof(tm));
   const char *s = strptime(datetime.c_str(), format.c_str(), &tm);
   if (s && *s) {
     cerr << "failed to parse date/time" << std::endl;
@@ -2463,69 +2426,81 @@ int RGWRados::process_intent_log(rgw_bucket& bucket, string& oid,
 				 time_t epoch, int flags, bool purge)
 {
   cout << "processing intent log " << oid << std::endl;
-  uint64_t size;
   rgw_obj obj(bucket, oid);
-  int r = obj_stat(NULL, obj, &size, NULL, NULL, NULL);
-  if (r < 0) {
-    cerr << "error while doing stat on " << bucket << ":" << oid
-	 << " " << cpp_strerror(-r) << std::endl;
-    return -r;
-  }
-  bufferlist bl;
-  r = read(NULL, obj, 0, size, bl);
-  if (r < 0) {
-    cerr << "error while reading from " <<  bucket << ":" << oid
-	 << " " << cpp_strerror(-r) << std::endl;
-    return -r;
-  }
 
-  bufferlist::iterator iter = bl.begin();
+  unsigned chunk = 1024 * 1024;
+  off_t pos = 0;
+  bool eof = false;
   bool complete = true;
-  try {
-    while (!iter.end()) {
-      struct rgw_intent_log_entry entry;
-      try {
-        ::decode(entry, iter);
-      } catch (buffer::error& err) {
-        dout(0) << "ERROR: " << __func__ << "(): caught buffer::error" << dendl;
-        return -EIO;
-      }
-      if (entry.op_time.sec() > epoch) {
-        cerr << "skipping entry for obj=" << obj << " entry.op_time=" << entry.op_time.sec() << " requested epoch=" << epoch << std::endl;
-        cerr << "skipping intent log" << std::endl; // no use to continue
-        complete = false;
-        break;
-      }
-      switch (entry.intent) {
-      case DEL_OBJ:
-        if (!flags & I_DEL_OBJ) {
-          complete = false;
-          break;
-        }
-        r = rgwstore->delete_obj(NULL, entry.obj);
-        if (r < 0 && r != -ENOENT) {
-          cerr << "failed to remove obj: " << entry.obj << std::endl;
-          complete = false;
-        }
-        break;
-      case DEL_POOL:
-        if (!flags & I_DEL_POOL) {
-          complete = false;
-          break;
-        }
-        r = delete_bucket(entry.obj.bucket);
-        if (r < 0 && r != -ENOENT) {
-          cerr << "failed to remove pool: " << entry.obj.bucket.pool << std::endl;
-          complete = false;
-        }
-        break;
-      default:
-        complete = false;
+  int ret = 0;
+  int r;
+
+  bufferlist bl;
+  bufferlist::iterator iter;
+  off_t off;
+
+  while (!eof || !iter.end()) {
+    off = iter.get_off();
+    if (!eof && (bl.length() - off) < chunk / 2) {
+      bufferlist more;
+      r = read(NULL, obj, pos, chunk, more);
+      if (r < 0) {
+        cerr << "error while reading from " <<  bucket << ":" << oid
+	   << " " << cpp_strerror(-r) << std::endl;
+        return -r;
       }
+      eof = (more.length() < chunk);
+      pos += more.length();
+      bufferlist old;
+      old.substr_of(bl, off, bl.length() - off);
+      bl.clear();
+      bl.claim(old);
+      bl.claim_append(more);
+      iter = bl.begin();
+    }
+    
+    struct rgw_intent_log_entry entry;
+    try {
+      ::decode(entry, iter);
+    } catch (buffer::error& err) {
+      cerr << "failed to decode intent log entry in " << bucket << ":" << oid << std::endl;
+      cerr << "skipping log" << std::endl; // no use to continue
+      ret = -EIO;
+      complete = false;
+      break;
+    }
+    if (entry.op_time.sec() > epoch) {
+      cerr << "skipping entry for obj=" << obj << " entry.op_time=" << entry.op_time.sec() << " requested epoch=" << epoch << std::endl;
+      cerr << "skipping log" << std::endl; // no use to continue
+      complete = false;
+      break;
+    }
+    switch (entry.intent) {
+    case DEL_OBJ:
+      if (!flags & I_DEL_OBJ) {
+        complete = false;
+        break;
+      }
+      r = rgwstore->delete_obj(NULL, entry.obj);
+      if (r < 0 && r != -ENOENT) {
+        cerr << "failed to remove obj: " << entry.obj << std::endl;
+        complete = false;
+      }
+      break;
+    case DEL_POOL:
+      if (!flags & I_DEL_POOL) {
+        complete = false;
+        break;
+      }
+      r = delete_bucket(entry.obj.bucket);
+      if (r < 0 && r != -ENOENT) {
+        cerr << "failed to remove pool: " << entry.obj.bucket.pool << std::endl;
+        complete = false;
+      }
+      break;
+    default:
+      complete = false;
     }
-  } catch (buffer::error& err) {
-    cerr << "failed to decode intent log entry in " << bucket << ":" << oid << std::endl;
-    complete = false;
   }
 
   if (complete) {
@@ -2538,5 +2513,5 @@ int RGWRados::process_intent_log(rgw_bucket& bucket, string& oid,
     }
   }
 
-  return 0;
+  return ret;
 }
