@@ -219,10 +219,11 @@ int rgw_read_user_buckets(string user_id, RGWUserBuckets& buckets, bool need_sta
     bufferlist bl;
 #define LARGE_ENOUGH_LEN (4096 * 1024)
     size_t len = LARGE_ENOUGH_LEN;
+    off_t ofs = 0;
     rgw_obj obj(ui_uid_bucket, buckets_obj_id);
 
     do {
-      ret = rgwstore->read(NULL, obj, 0, len, bl);
+      ret = rgwstore->read(NULL, obj, ofs, len, bl);
       if (ret == -ENOENT) {
         /* try to read the old format */
         ret = rgw_read_buckets_from_attr(user_id, buckets);
@@ -237,9 +238,10 @@ int rgw_read_user_buckets(string user_id, RGWUserBuckets& buckets, bool need_sta
       if (ret < 0)
         return ret;
 
-      if ((size_t)ret != len)
+      if (bl.length() != len)
         break;
 
+      bl.clear();
       len *= 2;
     } while (1);
 
